'******************************************************************************************
'======================= QChart:  A Charting Object for Rapid-Q ===========================
'====================== Copyright (c) 2003-2004  Michael J. Zito ==========================
'QChart provides a set of 2-D charting routines for Rapid-Q programmers.
'It generates histograms, bar charts, XY-scatter graphs, line graphs
'and box plots.  Each charting routine will calculate an appropriate scale
'and plot the data contained in the .Data component of the object.  The user may optionally
'specify the scaling as well as fonts, grids or other properties (see code)
'Once the data is formatted graphs can be called with very few lines of simple code:
'             .Initialize
'             .ChartType = ctXXXX
'             .ChartStyle = csXXXX
'     ------->Change Desired Options Here
'     ------->Dimension .Data Grid Here
'     ------->Load Data Into .Data Grid Here
'             .DrawChart (Overlay T or F)
'SEE QChartEx.rqb for examples of setting up and calling each chart type.
'
'NOTE: There is little error checking in this object.  It is up to the user
'to insure that the data is formatted properly before calling each routine.
'The .Data component is a non-visible QStringGrid which offers the user the
'ability to set the row and column count prior to filling the grid with data.
'The expected data formats for each chart type are listed below:
'
'Bar and Line Charts:  Data groups are in columns, data values are in each row
'                      Set .Data.ColCount to the number of groups
'                      Set .Data.RowCount to the number of data points in the largest group
'                      Row 0 is for legend text
'                      Col 0 is for X Axis Labels
'
'                          0   1   2  .   .   .   n
'                      0  Min <<======Legend======>>
'                      1   X   A1  B1 .   .   .   n1
'                      2   =   A2  B2 .   .   .   n2
'                      .   L   .   .  .   .   .   .
'                      .   b   .   .  .   .   .   .
'                      .   l   .   .  .   .   .   .
'                      n   s   An  Bn .   .   .   nn
'
'XY Scatter Charts:    Data pairs are in adjacent columns: Col1 = X val, Col2 = Y val
'                      Data values are in each row
'                      Set .Data.ColCount to the number of XY pairs * 2
'                      Set .Data.RowCount to the number of data points in the largest group
'                      Row 0 of the Y Col is for group labels / legend text
'                      Col 0 and Row 0 of the X Col are not used (*)
'                      Data for each pair should be in ascending order on the X value
'
'                               Pair 1      Pair 2        Pair n
'                               =======     =======       ======
'                          0    1     2     3     4  . . . m   n
'                      0   *    *    Lgd    *    Lgd       *  Lgd
'                      1   *   x11   y11   x21   y21 . . .xm1 yn1
'                      2   *   x12   y12   x22   y22 . . .xm2 yn2
'                      .   *    .     .     .     .  . . . .   .
'                      .   *    .     .     .     .  . . . .   .
'                      .   *    .     .     .     .  . . . .   .
'                      n   *   x1n   y1n   x2n   y2n . . .xmn ynn
'
'Box Plots:            Data groups are in columns, data values are in each row
'                      Set .Data.ColCount to the number of groups
'                      Set .Data.RowCount to 5
'                      Data must be set up with the highest value in row 1
'                      the lowest value in row 5 and intermediate values in
'                      descending order from row 1 to row 5.
'                      Row 0 is for legend text
'                      Col 0 is not used (*)
'
'                              Box Plots
'                          0   1   2  3   .   .   n
'                      0   *   <<=====Legend======>>
'                      1   *   Max
'                      2   *   3rd Quartile
'                      3   *   2nd Quartile (Median)
'                      4   *   1st Quartile
'                      5   *   Min
'
'Pie Charts            Data groups are in columns, data values are in Row 1
'                      Set .Data.ColCount to the number of groups
'                      Set .Data.RowCount to 1
'                      Row 0 is for legend text
'                      Col 0 is not used (*)
'
'                              Pie Charts
'                          0   1    2   3   .   .   n
'                      0   *   <<======Legend=======>>
'                      1   *   x1  x2  x3   .   .   xn
'
'******************************************************************************************

'----- Compiler Directives
     $IFNDEF TRUE
      $DEFINE TRUE 1
     $ENDIF
     $IFNDEF FALSE
      $DEFINE FALSE 0
     $ENDIF

'----- Win API Functions

'     DECLARE FUNCTION CreatePen LIB "gdi32" ALIAS "CreatePen" (nPenStyle AS LONG, nWidth AS LONG, crColor AS LONG)_
'      AS LONG
'     DECLARE FUNCTION CreateSolidBrush LIB "gdi32" ALIAS "CreateSolidBrush" (ByVal crColor AS LONG) AS LONG
'     DECLARE FUNCTION CreateHatchBrush LIB "gdi32" ALIAS "CreateHatchBrush" (nIndex AS LONG, crColor AS LONG) AS LONG
'     DECLARE FUNCTION SelectObject LIB "gdi32" ALIAS "SelectObject" (hdc AS LONG,hObject AS LONG) AS LONG
'     DECLARE FUNCTION DeleteObject LIB "gdi32" ALIAS "DeleteObject" (hObject AS LONG) AS LONG
'     DECLARE FUNCTION SetBkColor LIB "gdi32" ALIAS "SetBkColor" (hdc AS LONG, crColor AS LONG) AS LONG
'     DECLARE FUNCTION MoveToEx LIB "gdi32" ALIAS "MoveToEx" (hdc AS LONG,x AS LONG,y AS LONG, lpPoint AS LONG) AS LONG
'     DECLARE FUNCTION LineTo LIB "gdi32" ALIAS "LineTo" (hdc AS LONG,x AS LONG,y AS LONG) AS LONG
'     DECLARE FUNCTION Rectangle LIB "gdi32" ALIAS "Rectangle" (hdc AS LONG, X1 AS LONG, Y1 AS LONG, X2 AS LONG,_
'      Y2 AS LONG) AS LONG
'     DECLARE FUNCTION Pie LIB "gdi32" ALIAS "Pie" (ByVal hdc AS LONG, ByVal X1 AS LONG, ByVal Y1 AS LONG,_
'      ByVal X2 AS LONG, ByVal Y2 AS LONG, ByVal X3 AS LONG,_
'      ByVal Y3 AS LONG, ByVal X4 AS LONG, ByVal Y4 AS LONG)_
'      AS LONG

'----- PRIVATE QChart FUNCTIONs
'     DECLARE FUNCTION Log10 (Value AS SINGLE) AS SINGLE
     DECLARE FUNCTION RowTotal (Row AS INTEGER) AS SINGLE

'----- PRIVATE QChart SUBs
     DECLARE SUB ClearBuffer
     DECLARE SUB SetSize
     DECLARE SUB ScaleAxis (Mn AS DOUBLE, Mx AS DOUBLE, Axis AS INTEGER)
     DECLARE SUB SetScale
     DECLARE SUB SetSeries
     DECLARE SUB DrawAxes
     DECLARE SUB DrawTitles
     DECLARE SUB DrawLabels
     DECLARE SUB DrawMarker (x1 AS SINGLE, y1 AS SINGLE, MrkrStyle AS INTEGER, COLOR AS LONG)
     DECLARE SUB DrawLegend
     DECLARE SUB DrawBar (Overlay AS INTEGER)
     DECLARE SUB DrawXY (Overlay AS INTEGER)
     DECLARE SUB DrawLine (Overlay AS INTEGER)
     DECLARE SUB DrawBox (Overlay AS INTEGER)
     DECLARE SUB DrawPie (Overlay AS INTEGER)     
     DECLARE SUB drawp(type as integer,xb AS integer, yb AS integer,xc AS integer, yc AS integer,xd AS integer, yd AS integer)
	DECLARE SUB drawtrendline(xb AS SINGLE, yb AS SINGLE,xc AS SINGLE, yc AS SINGLE)
	DECLARE SUB drawfibofan(xb AS SINGLE, yb AS SINGLE,xc AS SINGLE, yc AS SINGLE)
DEFSNG rad = 3.1415927/180 ' to convert to radians

'----- PUBLIC QChart Methods
     DECLARE SUB Initialize
     DECLARE SUB PaintChart
     DECLARE SUB ClearAll
     DECLARE SUB DrawChart (Overlay AS INTEGER)
     DECLARE SUB RedrawChart
     DECLARE SUB SaveChart (FileNam AS STRING)
     DECLARE SUB PrintChart (Prn AS INTEGER, Orient AS INTEGER, Margin AS INTEGER,_
      Copies AS INTEGER, Colr AS BYTE)
     declare sub savebuffertmp
     declare sub restorebuffertmpz
     declare sub restorebuffertmp
	declare sub savebuffertmpsimple
	declare sub savebuffertmpsimplez
	declare sub bufcntreset
	declare sub firstbufcntreset

'----- PUBLIC variables
dim spaceforwards as integer
spaceforwards=0
dim bufcnt as integer
bufcnt=0
dim firstbufcnt as integer
firstbufcnt=0
dim logamplitude as double
logamplitude=5

'----- UDTs
     TYPE AxisType
      LEN           AS SINGLE               'Axis length in pixels
      Ori           AS SINGLE               'Logical Origin in pixels
      Min           AS DOUBLE               'Minimum scale value
      Max           AS DOUBLE               'Maximum scale value
      Div           AS INTEGER              'Number of divisions on axis
      ScaleFactor   AS SINGLE               'Factor for scaling values on axis
      AutoScale     AS INTEGER              'TRUE = QChart scales, FALSE = User supplies Min/Max
      COLOR         AS LONG                 'Color of axis
      Grid          AS INTEGER              'TRUE = Draw Grid, FALSE = No Grid
      Labeled       AS INTEGER              'TRUE = Label the Axis, FALSE = No Axis Labels
      Zero          AS SINGLE               'Location of zero in Pixels
      DrawZero      AS INTEGER              'TRUE = Draw Zero Line, FALSE = No Zero Line
      ZeroColor     AS LONG                 'Color of Zero Line
      Tics          AS INTEGER              'TRUE = Draw Tic Marks, FALSE = No Tic Marks
      MnrTics       AS INTEGER              'TRUE = Draw Minor Tic Marks, FALSE = No Tic Marks
      STEP          AS SINGLE               'Step increment value of each tic
      TicIntvl      AS SINGLE               'Distance between tics in world coordinates
      TicLen        AS SINGLE               'Size of tic mark in Pixels
      LogScale      AS INTEGER              'TRUE = Log Scalin, FALSE = Linear Scaling
     END TYPE

     TYPE TitleType
      Top           AS INTEGER
      Left          AS INTEGER
      Text          AS STRING * 70
     END TYPE

     TYPE SeriesType
      AutoColor     AS INTEGER              'Cycle available colors (T/F)
      COLOR         AS LONG                 'Color to use
      AutoMark      AS INTEGER              'Cycle available markers (T/F)
      Marker        AS INTEGER              'Marker style to use
      HatchStyle    AS LONG                 'Hatch Pattern to Use for Bars and Boxes
      LineStyle     AS LONG                 'Line Style to use
      LineWidth     AS LONG                 'Line Width to use
     END TYPE

' ---- For logspiral
type pointstype
        r as double
        x as double
        y as double
end type 
    
dim points(0 to 500) as pointstype
' ----

'----- Win API Constants
     CONST lsSolid=0                         'line styles
     CONST lsDash=1
     CONST lsDot=2
     CONST lsDashDot=3
     CONST lsDashDotDot=4
                                        'hatch styles
     CONST hsHoriz = 0                       '  -----
     CONST hsVert = 1                        '  |||||
     CONST hsFDiag = 2                       '  \\\\\
     CONST hsBDiag = 3                       '  /////
     CONST hsCross = 4                       '  +++++
     CONST hsXDiag = 5                       '  xxxxx
     CONST hsSolid = -1

'----- QChart Constants
     CONST ctBar = 0                         'Chart Type Flags
     CONST ctLine = 1
     CONST ctXY = 2
     CONST ctBox = 3
     CONST ctPie = 4

     CONST csHisto = 0                       'Bar Chart Style Flags
     CONST csBar = 1
     CONST csStacked = 2
     CONST csPctStacked = 3

     CONST csPoints = 4                      'XY and Line Chart Style Flags
     CONST csLines = 5
     CONST csBoth =6

     CONST csWhisker = 7                     'Box Plot Style Flags
     CONST csHiLo = 8

     CONST csPiePct = 9                         'Pie Chart Style Flags
     CONST csPieVal = 10

     CONST msFillCircle = 1                  'Marker Style Flags
     CONST msFillSquare = 2
     CONST msFillTriangleUp = 3
     CONST msFillTriangleDown = 4
     CONST msCircle = 5
     CONST msSquare = 6
     CONST msTriangleUp = 7
     CONST msTriangleDown = 8

     CONST MaxSeries = 1000                    'Maximum Number of Series that can be plotted

'----- Begin Object Definition ***********************************************************

     TYPE QChart EXTENDS Qcanvas

'---- Properties
PRIVATE:
      ChartExists         AS BYTE                 'TRUE or FALSE
      Missing             AS STRING               'Value used to test for missing data
      Success             AS BYTE                 'TRUE or FALSE; set in log routines
      Printing            AS BYTE                 'TRUE or FALSE; Update w/o screen redraw
      hPen                AS LONG                 'Handle to current pen
      OldPen              AS LONG                 'Handle to last used pen
      hBrush              AS LONG                 'Handle to current brush
      OldBrush            AS LONG                 'Handle to last used brush
      Buffer              AS QBITMAP              'Hidden drawing buffer
      Buffertmp              AS QBITMAP              'Hidden drawing buffer      
	buffertmpz as qbitmap
	buffertmpz2 as qbitmap
	firstbuffertmpz as qbitmap

PUBLIC:
      DATA                AS QSTRINGGRID          'Data to be plotted (See header notes for formats)
      DATA_INDI		  AS QSTRINGGRID          'Data to be plotted (See header notes for formats)
      XAxis               AS AxisType             'X-axis options
      YAxis               AS AxisType             'Y-axis options
      bgColor             AS LONG                 'background color
      fgColor             AS LONG                 'foreground color
      ChartType           AS BYTE                 'ctBar, ctXY, ctBox etc...
      ChartStyle          AS BYTE                 'csHisto, csLines, csPoints, csBoth etc...
      MainTitle           AS TitleType            'MainTitle Text and Location
      MainFont            AS QFONT
      demofont as qfont
      demofont2 as qfont
      SubTitle            AS TitleType            'SubTitle Text and Location
      SubFont             AS QFONT
      XTitle              AS TitleType            'X Axis Title Text and Location
      YTitle              AS TitleType            'Y Axis Title Text and Location
      AxisLbl             AS TitleType            'Axis Label Text and Location
      AxisFont            AS QFONT
      DoLegend            AS BYTE                 'TRUE = Draw Legend, FALSE = No Legend
      Legend              AS TitleType            'Legend Text and Location
      LegendFont          AS QFONT
      ChartBorder         AS BYTE                 'TRUE = Draw Frame around entire chart, FALSE = No Frame
      AxisBorder          AS BYTE                 'TRUE = Draw Frame around Plot Area, FALSE = No Frame
      Colors(24)          AS LONG                 'Array of colors; QChart chooses colors from this array
      PlotAreaColor       AS LONG                 'Color of plot area background
      GridColor           AS LONG                 'Color of grid lines
      dlgSave             AS QSAVEDIALOG
      MarkerSize          AS SINGLE               'Sets size of drawn markers
      BW                  AS BYTE                 'TRUE = Use BW only, FALSE = Cycles Colors() array
      GreyScale           AS BYTE                 'TRUE = Use GreyScale, FALSE = Cycles Colors() array
      Series(MaxSeries)   AS SeriesType           'Series style info

      CONSTRUCTOR

       Parent =            QFORM
       Missing =           ""

    '--- Expand and/or Redefine any way you like
       Colors(0) = RGB(0,0,0)              '-- Black
       Colors(1) = RGB(0,0,160)            '-- Blue
       Colors(2) = RGB(160,0,0)            '-- Red
       Colors(3) = RGB(0,160,0)            '-- Green
       Colors(4) = RGB(0,160,160)          '-- Cyan
       Colors(5) = RGB(160,0,160)          '-- Magenta
       Colors(6) = RGB(192,192,0)          '-- Yellow
       Colors(7) = RGB(0,0,255)            '-- Light Blue
       Colors(8) = RGB(255,0,0)            '-- Light Red
       Colors(9) = RGB(0,255,0)            '-- Light Green
       Colors(10) = RGB(0,255,255)         '-- Light Cyan
       Colors(11) = RGB(255,0,255)         '-- Light Magenta
       Colors(12) = RGB(255,255,0)         '-- Light Yellow
       Colors(13) = RGB(128,128,255)       '-- Torquoise
       Colors(14) = RGB(240,128,128)       '-- Rose Red
       Colors(15) = RGB(192,220,192)       '-- Money Green
       Colors(16) = RGB(166,202,240)       '-- Sky Blue
       Colors(17) = RGB(199,177,255)       '-- Lavender
       Colors(18) = RGB(255,226,177)       '-- Peach
       Colors(19) = RGB(255,153,51)        '-- Orange
       Colors(20) = RGB(255,51,153)        '-- Pink
       Colors(21) = RGB(80,80,80)          '-- Dark Grey
       Colors(22) = RGB(128,128,128)       '-- Med Grey
       Colors(23) = RGB(208,208,208)       '-- Light Grey
       Colors(24) = RGB(255,255,255)       '-- Bright White

      END CONSTRUCTOR

'------------------------------------------------------------------------------------------
'------------------------- Private Functions ----------------------------------------------
'------------------------------------------------------------------------------------------
PRIVATE:
      FUNCTION RowTotal (Row AS INTEGER) AS SINGLE

       DIM i AS INTEGER
       DIM Sum AS SINGLE

       WITH QChart
        Sum = 0
        FOR i = 1 TO .Data.ColCount
         IF .Data.Cell(i,Row) <> .Missing THEN
          Sum = Sum + ABS(VAL(.Data.Cell(i,Row)))
         END IF
        NEXT
       END WITH
       RowTotal = Sum

      END FUNCTION
'------------------------------------------------------------------------------------------
PRIVATE:
      FUNCTION Log10 (Value AS SINGLE) AS SINGLE

       IF Value > 0 THEN
        Log10 = LOG(Value) / LOG(10)
       ELSE
	Log10 = 0
'        MESSAGEDLG("Log10: Log of Value <= 0 Undefined", 1, 4, 0)
'        QChart.Success = FALSE
'        EXIT FUNCTION
       END IF

      END FUNCTION
'------------------------------------------------------------------------------------------
PRIVATE:
      SUB ClearBuffer                       'Erase off screen drawing buffer

       WITH QChart.Buffer
        .FillRect(0, 0, .Width, .Height, QChart.bgColor)
       END WITH
       QChart.ChartExists = FALSE          'Set flag

      END SUB
'------------------------------------------------------------------------------------------

private:
sub restorebuffertmp
with qchart
.buffer.bmp=.buffertmp.bmp
end with
end sub

'------------------------- Public Methods -------------------------------------------------
'------------------------------------------------------------------------------------------
PUBLIC:
      SUB Initialize                        'Sets default values, make global changes here

       DIM i AS INTEGER                    'Loop Counters
       DIM j AS INTEGER





       WITH QChart

        .Buffer.Width =     .Width
        .Buffer.Height =    .Height

        .bgColor =          .Colors(24)   'Bright White
        .fgColor =          .Colors(0)    'Black
        .PlotAreaColor =    plotareacolor '.Colors(23)   'Light Grey
        .GridColor =        .Colors(21)   'Dark Grey



        .ChartExists =      FALSE
        .Success =          TRUE
        .ChartBorder =      TRUE
        .AxisBorder =       TRUE
        .MarkerSize =       3
        .BW =               FALSE
        .GreyScale =        FALSE
        .DoLegend =         TRUE
        .Printing =         FALSE

        .XAxis.Div =        10
        .XAxis.AutoScale =  TRUE
        .XAxis.Labeled =    TRUE
        .XAxis.Tics =       TRUE
        .XAxis.MnrTics =    TRUE
        .XAxis.Grid =       TRUE
        .XAxis.DrawZero =   TRUE
        .XAxis.LogScale =   FALSE
        .XAxis.Color =      0
        .XAxis.ZeroColor =  QChart.Colors(8)'Red

        .YAxis.Div =        10
        .YAxis.AutoScale =  TRUE
        .YAxis.Labeled =    TRUE
        .YAxis.Tics =       TRUE
        .YAxis.MnrTics =    TRUE
        .YAxis.Grid =       TRUE
        .YAxis.DrawZero =   TRUE
        .YAxis.LogScale =   FALSE
        .YAxis.Color =      0
        .YAxis.ZeroColor =  QChart.Colors(8)'Red

        .MainFont.Name =    "Times New Roman"
        .MainFont.Size =    18
        .MainFont.Color =   QChart.Colors(0)
        .MainFont.AddStyles(0)            'Bold
	
	.demofont.name="Arial"
	.demofont.Size =    36
        .demofont.Color =   QChart.Colors(0)
        .demofont.AddStyles(0)            'Bold

	.demofont2.name="Arial"
	.demofont2.Size =    10
        .demofont2.Color =   QChart.Colors(0)
        .demofont2.AddStyles(0)            'Bold

        .SubFont.Name =     "Times New Roman"
        .SubFont.Size =     12
        .SubFont.Color =    QChart.Colors(0)
        .SubFont.AddStyles(0)            'Bold

        .AxisFont.Name =    "Times New Roman"
        .AxisFont.Size =    8
        .AxisFont.Color =   QChart.Colors(0)

        .LegendFont.Name =  "Times New Roman"
        .LegendFont.Size =  9
        .LegendFont.Color = QChart.Colors(0)

        FOR i = 1 TO MaxSeries            'Set Series Defaults
         .Series(i).AutoColor =  TRUE
         .Series(i).Color =      .fgColor
         .Series(i).AutoMark =   TRUE
         .Series(i).Marker =     msFillCircle
         .Series(i).HatchStyle = hsSolid
         .Series(i).LineStyle =  lsSolid
         .Series(i).LineWidth =  1
        NEXT

        FOR i = 0 TO .Data.ColCount       'Clear any residual data from last call
         FOR j = 0 TO .Data.RowCount
          .Data.Cell(i,j) = ""
         NEXT
        NEXT

        .MainTitle.Text =   ""
        .SubTitle.Text =    ""
        .XTitle.Text =      ""
        .YTitle.Text =      ""

        .Data.ColCount = 1                'Set .Data to minimum size
        .Data.RowCount = 1

        .ChartType = ctXY                 'Set default chart type and style
        .ChartStyle = csPoints

       END WITH

      END SUB
'------------------------------------------------------------------------------------------
PUBLIC:
      SUB PaintChart                        'Assign this to the QChart.OnPaint Event
                                        'Draws Buffer.BMP to Canvas
       WITH QChart
        .Draw(0, 0, .Buffertmp.BMP)
       END WITH

      END SUB
'------------------------------------------------------------------------------------------
PUBLIC:
      SUB ClearAll

       WITH QChart
        .Initialize
        .ClearBuffer
        .PaintChart
       END WITH

      END SUB
'------------------------------------------------------------------------------------------
'------------------------- Private Methods ------------------------------------------------
'------------------------------------------------------------------------------------------
PRIVATE:
      SUB SetSize

       DIM i AS INTEGER
       DIM LegendLen AS INTEGER

       WITH QChart                        'Set Origin and Axis Lengths
                                       'relative to window and font sizes
        .Buffer.Font = .AxisFont
        .XAxis.Ori = .Buffer.TextWidth("8") * 16
        IF .ChartType = ctPie THEN       'Leave room for right wedge labels
         .Xaxis.Ori = .XAxis.Ori / 2
        END IF
        IF .DoLegend = FALSE THEN        'Make X Axis larger
         .XAxis.Len = .Buffer.Width - .XAxis.Ori * 1.5-spaceforwards
        ELSE                             'Leave room for Legend
         .Buffer.Font = .LegendFont
         LegendLen = 0
         FOR i = 1 TO .Data.ColCount    'Get the longest legend text
          IF .Buffer.TextWidth(.Data.Cell(i,0)) > LegendLen THEN
           LegendLen = .Buffer.TextWidth(.Data.Cell(i,0))
          END IF
         NEXT i
         LegendLen = LegendLen + .Buffer.TextWidth("WWWWI")'Pad for marker space
         .XAxis.Len = .Buffer.Width - (.XAxis.Ori + LegendLen)
        END IF
        .Buffer.Font = .AxisFont
        .YAxis.Ori = .Buffer.Height - .Buffer.TextHeight(.XTitle.Text) * 5
        IF .ChartType = ctPie THEN
         .YAxis.Ori = .Buffer.Height - .Buffer.TextHeight(.XTitle.Text) * 3
        END IF
        .Buffer.Font = .MainFont
        .YAXis.Len = (.YAxis.Ori  - .Buffer.TextHeight(.MainTitle.Text) * 2) - 10

       END WITH

      END SUB
'------------------------------------------------------------------------------------------
PRIVATE:
      SUB ScaleAxis (Mn AS DOUBLE, Mx AS DOUBLE, Axis AS INTEGER)

       DIM prec AS DOUBLE
       DIM Incr AS DOUBLE
       DIM Div AS INTEGER
       DIM Range AS DOUBLE
       DIM i AS INTEGER
       DIM j AS DOUBLE
       DIM Test AS DOUBLE
       DIM pwr AS INTEGER
       DIM LMn1 AS DOUBLE
       DIM LMn2 AS DOUBLE
       DIM LMx1 AS DOUBLE
       DIM LMx2 AS DOUBLE
       DIM SclMn AS SINGLE
       DIM SclMx AS SINGLE

       WITH QChart

        IF Axis = 0 THEN Div = .XAxis.Div ELSE Div = .YAxis.Div 'Set values for desired axis

        IF Mx <= Mn THEN Mx = Mn + 1
        Range = Mx - Mn
        IF Mn >= 0 THEN
         Mn = 0
        ELSE
         Mn = Mn - Range * .01               'Prevent plotting points on chart border
        END IF
        IF Mx <= 0 THEN
         Mx = 0
        ELSE
         Mx = Mx + Range * .01               'Prevent plotting points on chart border
        END IF

        prec = .00002
        Range = (Mx - Mn) / Div
        pwr = FIX(.Log10(Range))
        IF Range < 1 THEN pwr = pwr - 1

        Incr = Range / (10^pwr)               'Calculate an even increment
        FOR j = 1 TO 5
         IF Incr < j + prec THEN EXIT FOR
        NEXT j

        FOR i  = j TO 20  '10?
         Test = i * 10^pwr

         LMn1 = Mn / Test
         LMn2 = FIX(LMn1)
         IF LMn1 < 0 THEN LMn2 = LMn2 - 1
         IF ABS(LMn2 + 1 - LMn1) < prec THEN LMn2 = LMn2 + 1
         SclMn = Lmn2 * Test

         LMx1 = Mx / Test
         LMx2 = FIX(LMx1 + 1)
         IF LMx1 < -1 THEN LMx2 = LMx2 - 1
         IF ABS(LMx1 + 1 - LMx2) < Prec THEN LMx2 = LMx2 - 1
         SclMx = LMx2 * Test

         Range = LMx2 - LMn2
         IF Range <= Div THEN EXIT FOR
        NEXT i

        Range = (Div - Range) / 2             'Set Scale min and max
        SclMn = SclMn - Range * Test
        SclMx = SclMn + Div * Test

        IF SclMn > Mn THEN SclMn = Mn         'Prevent errors
        IF SclMx < Mx THEN SclMx = Mx

        IF Mn >= 0 AND SclMn < 0 THEN         'Remove divisions where no data plotted
         SclMx = SclMx - SclMn               'so data takes up most of chart
         IF SclMx > Mx THEN
          Range = SclMx / Div
          WHILE SclMx > Mx + Range * 1.01
           SclMx = SclMx - Range
           Div = Div - 1
          WEND
         END IF
         SclMn = 0
        END IF

        IF Mx <= 0 AND SclMx > 0 THEN
         SclMn = SclMn - SclMx
         IF SclMn < Mn THEN
          Range = SclMn / Div
          WHILE SclMn < Mn + Range * 1.01
           SclMn = SclMn - Range
           Div = Div - 1
          WEND
         END IF
         SclMx = 0
        END IF

        IF sclMn * sclMx < 0 THEN
         Range = (SclMx - SclMn) / Div
         IF SclMn < Mn THEN
          WHILE SclMn < Mn - Range * 1.01
           SclMn = SclMn + Range
           Div = Div - 1
          WEND
         END IF
         IF SclMx > Mx THEN
          WHILE SclMx > Mx + Range * 1.01
           SclMx = SclMx - Range
           Div = Div - 1
          WEND
         END IF
        END IF

        SELECT CASE Axis
        CASE 0                              'Assign values to the X Axis
         .XAxis.Min = SclMn
         .XAxis.Max = SclMx
         .XAxis.Div = Div
         .XAxis.Step = (SclMx - SclMn) / Div
        CASE ELSE                           'Assign values to the Y Axis
         .YAxis.Min = SclMn
         .YAxis.Max = SclMx
         .YAxis.Div = Div
         .YAxis.Step = (SclMx - SclMn) / Div
        END SELECT

       END WITH

      END SUB
'------------------------------------------------------------------------------------------
PRIVATE:
      SUB SetScale

       DIM i    AS INTEGER                               'Loop counters...
       DIM j    AS INTEGER
       DIM XMax AS SINGLE
       DIM XMin AS SINGLE
       DIM YMax AS SINGLE
       DIM YMin AS SINGLE
       DIM Test AS SINGLE
       XMax = -1e35                                      'Start Small
       XMin = 1e35                                       'Start Big
       YMax = -1e35                                      'Ditto
       YMin = 1e35

       WITH QChart

        SELECT CASE .ChartType
        CASE ctBar, ctLine
         IF .YAxis.AutoScale = TRUE THEN             'Find Data Min and Max
          SELECT CASE .ChartStyle
          CASE csPctStacked
           .YAxis.Min = 0
           .YAxis.Max = 100
           .YAxis.Step = 10
           .YAxis.ScaleFactor = .YAxis.Len / 100
           EXIT SUB
          CASE csStacked
           FOR i = 1 TO .Data.RowCount
            Test = .RowTotal(i)
            IF Test < YMin THEN YMin = Test
            IF Test > YMax THEN YMax = Test
           NEXT i
          CASE ELSE
           FOR i = 1 TO .Data.ColCount
            FOR j = 1 TO .Data.RowCount
             IF .Data.Cell(i,j) <> .Missing THEN Test = VAL(.Data.Cell(i,j))
             IF Test < YMin THEN YMin = Test
             IF Test > YMax THEN YMax = Test
            NEXT j
           NEXT i
	
          END SELECT
         END IF

        CASE ctXY
         IF .XAxis.AutoScale = TRUE THEN             'Find  X Data Min and Max
          FOR i = 1 TO .Data.ColCount - 1 STEP 2
           FOR j = 1 TO .Data.RowCount
            IF .Data.Cell(i,j) <> .Missing THEN Test = VAL(.Data.Cell(i,j))
            IF Test < XMin THEN XMin = Test
            IF Test > XMax THEN XMax = Test
           NEXT j
          NEXT i
         END IF

         IF .YAxis.AutoScale = TRUE THEN             'Find Y Data Min and Max
          FOR i = 2 TO .Data.ColCount STEP 2
           FOR j = 1 TO .Data.RowCount
            IF .Data.Cell(i,j) <> .Missing THEN Test = VAL(.Data.Cell(i,j))
            IF Test < YMin THEN YMin = Test
            IF Test > YMax THEN YMax = Test
           NEXT j
          NEXT i
         END IF

        CASE ctBox
         IF .YAxis.AutoScale = TRUE THEN             'Find Data Min and Max
          FOR i = 1 TO .Data.ColCount
           IF .Data.Cell(i,5) <> .Missing THEN Test = VAL(.Data.Cell(i,5))
           IF Test < YMin THEN YMin = Test
           IF .Data.Cell(i,1) <> .Missing THEN Test = VAL(.Data.Cell(i,1))
           IF Test > YMax THEN YMax = Test
          NEXT i
         END IF
        END SELECT

        SELECT CASE .YAxis.LogScale
        CASE FALSE
         IF .YAxis.AutoScale = TRUE THEN
          .ScaleAxis (YMin, YMax, 1)
         ELSE
	graphcursorpos2
          IF .YAxis.Div < 1 THEN .YAxis.Div = 1
          .YAxis.Step = (.YAxis.Max - .YAxis.Min) / .YAxis.Div/yscaletrackbarposition
         END IF
         .YAxis.ScaleFactor = .YAxis.Len / (.YAxis.Max - .YAxis.Min)*yscaletrackbarposition
'Print .YAxis.ScaleFactor
         .YAxis.Zero = .YAxis.Ori + .YAxis.Min * .YAxis.ScaleFactor
' .yaxis.min=.yaxis.min*yscaletrackbarposition2 ' not implemented yet
        CASE TRUE                                  'See below for explanantion
         '.YAxis.Min = FLOOR(.Log10(YMin))
         '.YAxis.Max = CEIL(.Log10(YMax))
graphcursorpos2
         .YAxis.Min = FLOOR(.Log10(graphvlow))
         .YAxis.Max = CEIL(.Log10(graphvhigh))
         .YAxis.Div = (.YAxis.Max - .YAxis.Min)
         .YAxis.ScaleFactor = .YAxis.Len / .YAxis.Div*yscaletrackbarposition
         .YAxis.TicIntvl = .YAxis.ScaleFactor

        END SELECT

        IF .ChartType <> ctXY THEN EXIT SUB         'If not XY no Values on X Axis so Bail!

        SELECT CASE .XAxis.LogScale
        CASE FALSE
         IF .XAxis.AutoScale = TRUE THEN
          .ScaleAxis (XMin, XMax, 0)
         ELSE
          IF .XAxis.Div < 1 THEN .XAxis.Div = 1
          .XAxis.Step = (.XAxis.Max - .XAxis.Min) / .XAxis.Div
         END IF
         .XAxis.ScaleFactor = .XAxis.Len / (.XAxis.Max - .XAxis.Min)
'Print .XAxis.ScaleFactor
         .XAxis.Zero = .XAxis.Ori - .XAxis.Min * .XAxis.ScaleFactor
        CASE TRUE                                 'Log Scale
         .XAxis.Min = FLOOR(.Log10(XMin))          'Set Min to Lower power of ten
         .XAxis.Max = CEIL(.Log10(XMax))           'Set Max to Greater power of ten
         .XAxis.Div = (.XAxis.Max - .XAxis.Min)    'Div = # of Log Cycles
         .XAxis.ScaleFactor = .XAxis.Len / .XAxis.Div*3
         .XAxis.TicIntvl = .XAxis.ScaleFactor
        END SELECT

       END WITH

      END SUB
'------------------------------------------------------------------------------------------
PRIVATE:
      SUB SetSeries                              'Set up series colors and marker types

       DIM i AS INTEGER
       DIM NumSeries AS INTEGER
       DIM Grey AS INTEGER
       DIM NumShades AS INTEGER
       DIM White AS INTEGER

       WITH QChart
        IF .ChartType = ctXY THEN
         NumSeries = .Data.ColCount\2
        ELSE
         NumSeries = .Data.ColCount
        END IF

        IF .ChartType = ctBar OR .ChartType = ctBox OR .ChartType = ctPie THEN
         NumShades = 5
         White = 1
        ELSE
         NumShades = 4
         White = -1
        END IF

        FOR i = 1 TO NumSeries
         IF .Series(i).AutoMark=TRUE THEN .Series(i).Marker = (i-1) MOD 8 + 1
         IF .Series(i).AutoColor=TRUE THEN
          .Series(i).Color = .Colors((i) MOD (UBOUND(QChart.Colors) + White))
         END IF
         IF .GreyScale = TRUE THEN
          Grey = ((i-1) MOD NumShades) * 63
          .Series(i).Color = RGB(Grey,Grey,Grey)
         END IF
         IF .BW = TRUE THEN .Series(i).Color = i'0
        NEXT
       END WITH
      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawAxes

       DIM i  AS INTEGER                               'Loop counters...
       DIM j  AS INTEGER
       DIM x1 AS SINGLE                                'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE	

       WITH QChart

        x1 = 2                                        'Draw a frame around entire chart
        y1 = 2
        x2 = .Buffer.Width - 2
        y2 = .Buffer.Height - 2
        IF .ChartBorder = TRUE THEN .Buffer.Rectangle (x1, y1, x2, y2, .fgColor)

        IF .ChartType = ctPie THEN EXIT SUB           'No Axes for Pie Charts

        x1 = .XAxis.Ori                               'Draw a box around Plot Area
        y1 = .YAxis.Ori
        x2 = .XAxis.Ori + .XAxis.Len
        y2 = .YAxis.Ori - .YAxis.Len
        IF .AxisBorder = TRUE THEN
         .Buffer.Rectangle (x1, y1, x2 + 1, y2, .GridColor)
         .Buffer.FillRect (x1 + 1, y1 + 1, x2, y2 + 1, .PlotAreaColor)
        END IF

        IF .XAxis.Tics = TRUE THEN                    'Set Tic Size
         .XAxis.TicLen = .YAxis.Len * .01
        ELSE
         .XAxis.TicLen = 0
        END IF

        IF .XAxis.Grid = TRUE THEN                    'Draw a grid and tics
         y1 = .YAxis.Ori + .XAxis.TicLen
         y2 = .YAxis.Ori - .YAxis.Len
        ELSE                                          'Draw Tics only
         y1 = .YAxis.Ori + .XAxis.TicLen
         y2 = .YAxis.Ori - .Xaxis.TicLen
        END IF

        IF .XAxis.LogScale = FALSE THEN               'draw linear grid lines
         IF .XAxis.Div < 1 THEN .XAxis.Div = 1       'Avoid division by zero
         .XAxis.TicIntvl = .XAxis.Len / .XAxis.Div
         FOR i = 0 TO .XAxis.Div step 32
          x1 = .XAxis.Ori + .XAxis.TicIntvl * i
          x2 = x1
          .Buffer.Line (x1,y1,x2,y2,.GridColor)
         NEXT
         IF .ChartType = ctXY AND .XAxis.MnrTics = TRUE THEN
          y1 = .YAxis.Ori - .XAxis.TicLen / 2
          y2 = .YAxis.Ori + .XAxis.TicLen / 2
          FOR i = 0 TO .XAxis.Div - 1
           FOR j = 1 TO 4
            x1 = .XAxis.Ori + .XAxis.TicIntvl * i + .XAxis.TicIntvl/5 * j
            x2 = x1
            .Buffer.Line (x1,y1,x2,y2,.GridColor)
           NEXT
          NEXT
         END IF
        ELSE                                          'draw log grid lines
         FOR i = .XAxis.Min TO .XAxis.Max - 1
          FOR j  = 1 TO 9
           x1 = .XAxis.Ori + (.Log10(10 ^ i * j) - .XAxis.Min) * .XAxis.ScaleFactor
           x2 = x1
           .Buffer.Line (x1,y1,x2,y2,.GridColor)
          NEXT j
         NEXT i
         .Buffer.Line (.XAxis.Ori+.XAxis.Len,y1,.XAxis.Ori+.XAxis.Len,y2,.GridColor)
        END IF

        IF .YAxis.Tics = TRUE THEN                    'Set Tic Size
         .YAxis.TicLen = .XAxis.Len * .01
        ELSE
         .YAxis.TicLen = 0
        END IF

        IF .YAxis.Grid = TRUE THEN                    'Draw a grid and tics
         x1 = .XAxis.Ori - .YAxis.TicLen
         x2 = .XAxis.Ori + .XAxis.Len
        ELSE                                          'Draw Tics only
         x1 = .XAxis.Ori - .YAxis.TicLen
         x2 = .XAxis.Ori + .YAxis.TicLen
        END IF

        IF .YAxis.LogScale = FALSE THEN               'draw linear grid lines
         IF .YAxis.Div < 1 THEN .YAxis.Div = 1       'Avoid division by zero
         .YAxis.TicIntvl = .YAxis.Len / .YAxis.Div
         FOR i = 0 TO .YAxis.Div
          y1 = .YAxis.Ori - .YAxis.TicIntvl * i
          y2 = y1
          '.Buffer.Line (x1,y1,x2,y2,.GridColor)     ' horizontal grid lines
          .Buffer.Line (x1,y1,.XAxis.Ori + .YAxis.TicLen,y2,.GridColor)     ' y axis tics 
         NEXT
         IF .YAxis.MnrTics = TRUE THEN
          x1 = .XAxis.Ori - .YAxis.TicLen / 2
          x2 = .XAxis.Ori + .YAxis.TicLen / 2
          FOR i = 0 TO .YAxis.Div - 1
           FOR j = 1 TO 4
            y1 = .YAxis.Ori - .YAxis.TicIntvl * i - .YAxis.TicIntvl/5 * j
            y2 = y1
            .Buffer.Line (x1,y1,x2,y2,.GridColor)
           NEXT
          NEXT
         END IF


        ELSE                                          'draw log grid lines
         
FOR i = .YAxis.Min TO .YAxis.Max - 1
          FOR j  = 1 TO 9
           y1 = .YAxis.Ori - (.Log10(10 ^ i * j) - .YAxis.Min) * .YAxis.ScaleFactor
           y2 = y1
           .Buffer.Line (x1,y1,x2,y2,.GridColor)
          NEXT j
         NEXT i
         .Buffer.Line (x1,.YAxis.Ori-.YAxis.Len,x2,.YAxis.Ori-.YAxis.Len,.GridColor)

        END IF

        IF .XAxis.DrawZero = TRUE AND .XAxis.LogScale = FALSE THEN 'Draw the X Zero Line
         IF .XAxis.Min * .XAxis.Max < 0 THEN         'Data Spans zero
          x1 = .XAxis.Zero
          y1 = .YAxis.Ori
          x2 = .XAxis.Zero
          y2 = .YAxis.Ori - .YAxis.Len + 1
          .Buffer.Line (x1,y1,x2,y2,.XAxis.ZeroColor)
         END IF
        END IF

        IF .YAxis.DrawZero = TRUE AND .YAxis.LogScale = FALSE THEN 'Draw the Y Zero Line
         IF .YAxis.Min * .YAxis.Max < 0 THEN         'Data Spans zero
          x1 = .XAxis.Ori
          y1 = .YAxis.Zero
          x2 = .XAxis.Ori + .XAxis.Len - 1
          y2 = .YAxis.Zero
          .Buffer.Line (x1,y1,x2,y2,.YAxis.ZeroColor)
         END IF
        END IF

        x1 = .XAxis.Ori                               'Overwrite XAxis in its color
        y1 = .YAxis.Ori
        x2 = .XAxis.Ori + .XAxis.Len
        y2 = .YAxis.Ori
        .Buffer.Line (x1, y1, x2, y2, .XAxis.Color)

        x2 = .XAxis.Ori
        y2 = .YAxis.Ori - .YAxis.Len                  'Overwrite YAxis in its color
        .Buffer.Line (x1, y1, x2, y2, .YAxis.Color)

       END WITH

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawTitles

       DIM i    AS INTEGER                             'Loop counters...
       DIM j    AS INTEGER
       DIM Spacer AS INTEGER

       WITH QChart
      '--- Main Title
        .Buffer.Font = .MainFont
        .MainTitle.Left = .XAxis.Ori + (.XAxis.Len - .Buffer.TextWidth(RTRIM$(.MainTitle.Text))) / 2
        .MainTitle.Top = 3
        .Buffer.TextOut (.MainTitle.Left, .MainTitle.Top, RTRIM$(.MainTitle.Text),_
         .MainFont.Color, .bgColor)

      '--- Sub Title
        Spacer = .Buffer.TextHeight(.MainTitle.Text)  'Calc this before Font is Changed
        .Buffer.Font = .SubFont
        .SubTitle.Left = .XAxis.Ori + (.XAxis.Len  - .Buffer.TextWidth(RTRIM$(.SubTitle.Text))) / 2
        .SubTitle.Top = .MainTitle.Top + Spacer
        .Buffer.TextOut (.SubTitle.Left, .SubTitle.Top, RTRIM$(.SubTitle.Text),_
         .SubFont.Color, .bgColor)

      '--- X Axis Title
        .Buffer.Font = .AxisFont 'both x and y axis
        .XTitle.Left = .XAxis.Ori + (.XAxis.Len - .Buffer.TextWidth(RTRIM$(.XTitle.Text))) / 2
        .XTitle.Top =.YAxis.Ori -10+ .Buffer.TextHeight(.XTitle.Text) * 2.75
        .Buffer.TextOut (.XTitle.Left, .XTitle.Top, RTRIM$(.XTitle.Text),_
         .AxisFont.Color, .bgColor)

'--- Copyright mark
        .Buffer.Font = .AxisFont 'both x and y axis
        .XTitle.Left = .XAxis.Ori - 70
        .XTitle.Top =.YAxis.Ori -10+ .Buffer.TextHeight(.XTitle.Text) * 2.75
        .Buffer.TextOut (.XTitle.Left, .XTitle.Top, RTRIM$("QChartist (c) 2010-2011 Julien Moog - http://www.qchartist.com"),_
         .AxisFont.Color, .bgColor)

      '--- Y Axis Title
        j = LEN(RTRIM$ (.YTitle.Text))
        .YTitle.Left = .XAxis.Ori / 6
        .YTitle.Top = (.Buffer.Height - .Buffer.TextHeight(.YTitle.Text)*j) / 2
        FOR i = 1 TO j                  '--- Draw Y label vertically
         .Buffer.TextOut (.YTitle.Left, .YTitle.Top, MID$(.YTitle.Text, i ,1),_
          .AxisFont.Color, .bgColor)
         .YTitle.Top = .YTitle.Top + .Buffer.TextHeight(.YTitle.Text)
        NEXT i

       END WITH

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawLabels

       DIM i AS INTEGER
       DIM TicVal AS SINGLE                             'Value of tic label
       DIM LabelLen AS SINGLE                           'Total length of Labels

       WITH QChart

        IF .XAxis.Labeled = FALSE AND .YAxis.Labeled = FALSE THEN EXIT SUB

        .Buffer.Font = .AxisFont                       'Assign Legend font to buffer
        IF .XAxis.Div < 1 THEN .XAXis.Div = 1          'Avoid division by zero
        IF .YAxis.Div < 1 THEN .YAxis.Div = 1

        SELECT CASE .ChartType
        CASE ctXY
         IF .XAxis.Labeled THEN

          LabelLen = 0                              'Get the total length of all labels
          FOR i = 0 TO .XAxis.Div
           TicVal = .XAxis.Min + .XAxis.Step * i
           LabelLen = LabelLen + .Buffer.TextWidth(FORMAT$("%-4.4g", TicVal))
          NEXT

          FOR i = 0 TO .XAxis.Div
           IF .XAxis.LogScale = FALSE THEN
            TicVal = .XAxis.Min + .XAxis.Step * i 'below avoids precision artifact at 0
            IF ABS((.XAxis.Ori + .XAxis.TicIntvl * i) - .XAxis.Zero) < 1 THEN TicVal = 0
           ELSE                                    'Log Scale
            TicVal = 10^(i + .XAxis.Min)
           END IF
           .AxisLbl.Text = FORMAT$("%-4.4g", TicVal)
           .AxisLbl.Top = .YAxis.Ori + .Buffer.TextHeight(.AxisLbl.Text) / 2
           .AxisLbl.Left = .XAxis.Ori + .XAxis.TicIntvl * i -_
            .Buffer.TextWidth(RTRIM$(.AxisLbl.Text)) / 2
           IF LabelLen * 1.1 <= .XAxis.LEN THEN    'Label all tics
            .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top, RTRIM$(.AxisLbl.Text),_
             .Buffer.Font.Color, .bgColor)
           ELSE                                    'Label every other tic
            IF i MOD 2 = 0 THEN .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top,_
             RTRIM$(.AxisLbl.Text), .Buffer.Font.Color, .bgColor)
           END IF
          NEXT
         END IF

        CASE ctBox                                   'Labels in Row 0 of of each Col
         IF .XAxis.Labeled = TRUE THEN
          .XAxis.TicIntvl = .XAxis.Len / .Data.ColCount

          LabelLen = 0                             'Get the total length of all labels
          FOR i = 0 TO .Data.ColCount
           LabelLen = LabelLen + .Buffer.TextWidth(.Data.Cell(i,0))
          NEXT i

          FOR i = 1 TO .Data.ColCount step 32
           .AxisLbl.Text = .Data.Cell(i,0)
           .AxisLbl.Left = .XAxis.Ori + (i-1) * .Xaxis.TicIntvl + [.Xaxis.TicIntvl -_
           .Buffer.TextWidth(RTRIM$(.AxisLbl.Text))] / 2

           IF LabelLen * 1.1 <= .XAxis.Len THEN
            .AxisLbl.Top = .YAxis.Ori + .Buffer.TextHeight(.AxisLbl.Text) / 2
           ELSE                                    'alternate up and down for room
            .AxisLbl.Top = .YAxis.Ori + .Buffer.TextHeight(.AxisLbl.Text) *_
             (((i-1) MOD 2) + .5)
           END IF
           .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top,RTRIM$(.AxisLbl.Text),_
           .Buffer.Font.Color, .bgColor)
          NEXT i
         END IF

        CASE ELSE                                     'Labels in Col 0 of each Row
         IF .XAxis.Labeled = TRUE THEN
          SELECT CASE .ChartType
          CASE ctLine
           .XAxis.TicIntvl = .XAxis.Len / (.Data.RowCount - 1)
          CASE ELSE
           .XAxis.TicIntvl = .XAxis.Len / .Data.RowCount
          END SELECT

          LabelLen = 0                             'Get the total length of all labels
          FOR i = 0 TO .Data.RowCount
           LabelLen = LabelLen + .Buffer.TextWidth(.Data.Cell(0,i))
          NEXT i

          FOR i = 0 TO .Data.RowCount
           IF .ChartStyle = csHisto THEN          'Histo Min val in Col 0 of Row 0 ++
            .AxisLbl.Text = .Data.Cell(0,i)
           ELSE                                   'Labels stored in Col 0 of Row 1 ++
            .AxisLbl.Text = .Data.Cell(0,i+1)
           END IF

           IF .ChartType = ctLine OR .ChartStyle = csHisto THEN
            .AxisLbl.Left = .XAxis.Ori + (i) * .Xaxis.TicIntvl  - _
             .Buffer.TextWidth(RTRIM$(.AxisLbl.Text)) / 2
           ELSE
            .AxisLbl.Left = .XAxis.Ori + (i) * .Xaxis.TicIntvl + [.Xaxis.TicIntvl -_
             .Buffer.TextWidth(RTRIM$(.AxisLbl.Text))] / 2
           END IF

           IF LabelLen * 1.1 <= .XAxis.Len THEN
            .AxisLbl.Top = .YAxis.Ori + .Buffer.TextHeight(.AxisLbl.Text) / 2
           ELSE                                    'alternate up and down for room
            .AxisLbl.Top = .YAxis.Ori + .Buffer.TextHeight(.AxisLbl.Text) *_
             ((i MOD 2) + .5)
           END IF
           .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top,RTRIM$(.AxisLbl.Text),_
            .Buffer.Font.Color, .bgColor)
          NEXT i
         END IF
        END SELECT

        IF .YAxis.Labeled = TRUE THEN               'ALL Charts have values on the Y Axis
         FOR i = 0 TO .YAxis.Div
          IF .YAXIS.LogScale = FALSE THEN

           TicVal = .YAxis.Min + .YAxis.Step * i 'below avoids precision artifact at 0
           IF ABS((.YAxis.Ori - .YAxis.TicIntvl * i) - .YAxis.Zero) < 1 THEN TicVal = 0

          ELSE                                    'Log Scaling

           TicVal = 10^(i + .YAxis.Min)

          END IF

          .AxisLbl.Text =  FORMAT$("%-4.4g",TicVal)
          .AxisLbl.Top = .YAxis.Ori - .YAxis.TicIntvl * i_
           - .Buffer.TextHeight(.AxisLbl.Text) / 2
          .AxisLbl.Left = .XAxis.Ori - .Buffer.TextWidth(RTRIM$(.AxisLbl.Text))-_
           .Buffer.TextWidth("8")
          .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top,RTRIM$(.AxisLbl.Text),_
           .Buffer.Font.Color, .bgColor)
         NEXT
        END IF

       END WITH

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawMarker (x1 AS SINGLE, y1 AS SINGLE, MrkrStyle AS INTEGER, COLOR AS LONG)

       DIM x2 AS SINGLE
       DIM y2 AS SINGLE

       WITH QChart

        x1 = x1 - .MarkerSize                   'MarkerSize is set in .Intitalize or
        y1 = y1 - .MarkerSize                   'changed by user
        x2 = x1 + .MarkerSize * 2
        y2 = y1 + .MarkerSize * 2

        SELECT CASE MrkrStyle                   'Sent by calling routine

        CASE msFillCircle
         .Buffer.Circle (x1 , y1 , x2 , y2 , COLOR, COLOR)

        CASE msFillSquare
         .Buffer.FillRect (x1 , y1 , x2 , y2 , COLOR)

        CASE msFillTriangleUp
         .Buffer.Line (x1, y2, x2, y2, COLOR)
         .Buffer.Line (x1, y2 , x1 + .MarkerSize, y1, COLOR)
         .Buffer.Line (x2, y2 , x1 + .MarkerSize, y1, COLOR)
         .Buffer.Paint (x1 + .MarkerSize, y1 + .MarkerSize, COLOR, COLOR)

        CASE msFillTriangleDown
         y2 = y2 - .MarkerSize * 2
         .Buffer.Line (x1, y2, x2, y2, COLOR)
         .Buffer.Line (x1, y2 , x1 + .MarkerSize, y1 + .MarkerSize * 2, COLOR)
         .Buffer.Line (x2, y2 , x1 + .MarkerSize, y1 + .MarkerSize * 2, COLOR)
         .Buffer.Paint (x1 + .MarkerSize, y1 + .MarkerSize, COLOR, COLOR)

        CASE msCircle
         .Buffer.Circle (x1 , y1 , x2 , y2 , COLOR, .bgColor)

        CASE msSquare
         .Buffer.Rectangle (x1 , y1 , x2 , y2 , COLOR)
         .Buffer.FillRect (x1+1 , y1+1.1 , x2-1 , y2-1.1 , .bgColor)

        CASE msTriangleUp
         .Buffer.Line (x1, y2, x2, y2, COLOR)
         .Buffer.Line (x1, y2 , x1 + .MarkerSize, y1, COLOR)
         .Buffer.Line (x2, y2 , x1 + .MarkerSize, y1, COLOR)
         .Buffer.Paint (x1 + .MarkerSize, y1 + .MarkerSize, .bgColor, COLOR)

        CASE msTriangleDown
         y2 = y2 - .MarkerSize * 2
         .Buffer.Line (x1, y2, x2, y2, COLOR)
         .Buffer.Line (x1, y2 , x1 + .MarkerSize, y1 + .MarkerSize * 2, COLOR)
         .Buffer.Line (x2, y2 , x1 + .MarkerSize, y1 + .MarkerSize * 2, COLOR)
         .Buffer.Paint (x1 + .MarkerSize, y1 + .MarkerSize, .bgColor, COLOR)

        END SELECT

       END WITH

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawLegend

       DIM i  AS INTEGER                                   'Loop counter...
       DIM x1 AS SINGLE                                    'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE
       DIM BorderHgt AS INTEGER
       DIM LineLen AS INTEGER
       DIM LMark AS INTEGER

       WITH QChart
        IF .DoLegend = FALSE THEN EXIT SUB                           'Don't perform task

        .Buffer.Font = .LegendFont

        x1 = (.XAxis.Ori + .XAxis.Len) + .Buffer.TextWidth("W")      'Draw a box for the legend
        x2 = .Buffer.Width - .Buffer.TextWidth("W")
        LineLen = x2 -  x1 - .Buffer.TextWidth("W") * 2
        BorderHgt = .Buffer.TextHeight("W") * (.Data.ColCount) + 5
        IF .ChartType = ctXY THEN                                    'Special Case
         BorderHgt = .Buffer.TextHeight("W") * (.Data.ColCount / 2) + 5
        END IF
        IF .ChartStyle = csLines THEN BorderHgt = BorderHgt * 1.5     'Leave room for line styles
        y1 = .YAxis.Ori - (.YAxis.Len + BorderHgt) / 2
        y2 = y1 + BorderHgt
        .Buffer.Rectangle (x1,y1,x2,y2,.fgColor)

        SELECT CASE .ChartType

        CASE ctXY
         SetBkColor (.Buffer.Handle, .bgColor)

         FOR i = 2 TO .Data.ColCount STEP 2
          .Legend.Text = .Data.Cell(i, 0)
          .Legend.Top = y1 + .Buffer.TextHeight(.Legend.Text) * (i \ 2 - 1) + 2
          .Legend.Left = x1 + .Buffer.TextWidth("I")
          x2 = .Legend.Left + .Buffer.TextHeight(.Legend.Text)
          y2 = .Legend.Top + .Buffer.TextHeight(.Legend.Text)
          IF .ChartStyle <> csLines THEN
           LMark = .MarkerSize
           .MarkerSize = .Buffer.TextHeight(.Legend.Text) / 3
           .DrawMarker (.Legend.Left + .Buffer.TextHeight(.Legend.Text) / 3,_
            .Legend.Top + .Buffer.TextHeight(.Legend.Text) / 2,_
            .Series(i\2).Marker, .Series(i\2).Color)
           .Buffer.TextOut (x2 + 2, .Legend.Top, RTRIM$(.Legend.Text),_
            .LegendFont.Color, .bgColor)
           .MarkerSize = LMark
          ELSE
           .Legend.Top = .Legend.Top + .Buffer.TextHeight(.Legend.Text)/3 * (i\2-1)
           .hPen = CreatePen(.Series(i\2).LineStyle, .Series(i\2).LineWidth, .Series(i\2).Color)
           .OldPen = SelectObject(.Buffer.Handle, .hPen)
           MoveToEx(.Buffer.handle,.Legend.Left, y2 + .Buffer.TextHeight(.Legend.Text)/2*(i\2-1),0)
           LineTo(.Buffer.handle,x2 + LineLen, y2 + .Buffer.TextHeight(.Legend.Text)/2*(i\2-1))
           DeleteObject(SelectObject(.Buffer.Handle, .OldPen))
           .Buffer.TextOut (x2 - .Buffer.TextWidth("I") , .Legend.Top, RTRIM$(.Legend.Text),_
            .LegendFont.Color, .bgColor)
          END IF
         NEXT

        CASE ELSE                                       'Below works for all other charts
         SetBkColor (.Buffer.Handle, .bgColor)

         .Buffer.Line (0,0,0,0,0)                      'Reset Buffer's pen to Black
         FOR i = 1 TO .Data.ColCount
          .Legend.Text = .Data.Cell(i, 0)
          .Legend.Top = y1 + .Buffer.TextHeight(.Legend.Text) * (i-1) + 2
          .Legend.Left = x1 + .Buffer.TextWidth("I")
          x2 = .Legend.Left + .Buffer.TextHeight(.Legend.Text)
          y2 = .Legend.Top + .Buffer.TextHeight(.Legend.Text)
          IF .ChartType = ctLine THEN
           IF .ChartStyle <> csLines THEN
            LMark = .MarkerSize
            .MarkerSize = .Buffer.TextHeight(.Legend.Text) / 3
            .DrawMarker (.Legend.Left + .Buffer.TextHeight(.Legend.Text) / 3,_
             .Legend.Top + .Buffer.TextHeight(.Legend.Text) / 2,_
             .Series(i).Marker, .Series(i).Color)
            .Buffer.TextOut (x2 + 2, .Legend.Top, RTRIM$(.Legend.Text),_
             .LegendFont.Color, .bgColor)
            .MarkerSize = LMark
           ELSE
            .Legend.Top = .Legend.Top + .Buffer.TextHeight(.Legend.Text)/2 * (i-1)
            .hPen = CreatePen(.Series(i).LineStyle, .Series(i).LineWidth, .Series(i).Color)
            .OldPen = SelectObject(.Buffer.Handle, .hPen)
            MoveToEx(.Buffer.handle,.Legend.Left, y2 + .Buffer.TextHeight(.Legend.Text)/2*(i-1),0)
            LineTo(.Buffer.handle,x2 + LineLen, y2 + .Buffer.TextHeight(.Legend.Text)/2*(i-1))
            DeleteObject(SelectObject(.Buffer.Handle, .OldPen))
            .Buffer.TextOut (x2 - .Buffer.TextWidth("I") , .Legend.Top, RTRIM$(.Legend.Text),_
             .LegendFont.Color, .bgColor)
           END IF
          ELSE
           IF .Series(i).HatchStyle = hsSolid THEN
            .hBrush = CreateSolidBrush(.Series(i).Color)
           ELSE
            .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
           END IF
           .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
           Rectangle (.Buffer.Handle, .Legend.Left, .Legend.Top + 2, x2, y2)
           DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
           .Buffer.TextOut (x2 + 2, .Legend.Top, RTRIM$(.Legend.Text),_
            .LegendFont.Color, .bgColor)
          END IF

         NEXT

        END SELECT
       END WITH

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawBar (Overlay AS INTEGER)

  'X Points are (OriX + VAL * XScaleFactor - XMin) (ScaleFactor = Axis.Len / Axis.Max)
  'Y Points are (OriY - VAL * YScaleFactor - YMin) (i.e. ^^  = Pixels per Unit)

       DIM i  AS INTEGER                               'Loop counters...
       DIM j  AS INTEGER
       DIM x1 AS SINGLE                                'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE
       DIM GrpWidth AS SINGLE
       DIM BoxWidth AS SINGLE                          'Width of bars in bar and box plots
       DIM Incr AS SINGLE                              'Space between bars
       DIM Total AS SINGLE                             'Row Total for % Stacked calcs

       IF Overlay = FALSE THEN
        WITH QChart                                   'Set up the chart window
         .ClearBuffer                                'erase the current bitmap
         .SetSize
         .SetScale
         IF .Success = FALSE THEN                    'Log Scaling failed
          .ClearAll
          EXIT SUB
         END IF
         .SetSeries
         .XAxis.Div = .Data.RowCount                 'Forces a grid line for each X Label
         .XAxis.DrawZero = FALSE                     'No Values on the X
         .DrawAxes
         .DrawTitles
         .DrawLabels
         .DrawLegend
        END WITH
       END IF

       SELECT CASE QChart.ChartStyle

       CASE csHisto                                       'Histogram (no spaces between bars)
        WITH QChart                                      'Plot the data
         BoxWidth = (.XAxis.Len + .01) / .Data.RowCount 'Leave some space at axis
         FOR i = 1 TO .Data.ColCount
          IF .Series(i).HatchStyle = hsSolid THEN
           .hBrush = CreateSolidBrush(.Series(i).Color)
          ELSE
           .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
          END IF
          .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
          Incr = BoxWidth  *  (i-1) / (.Data.ColCount + 1)
          GrpWidth = BoxWidth - Incr
          FOR j = 1 TO .Data.RowCount
           IF .Data.Cell(i,j) = .Missing THEN GOTO NoPlotHisto
           x1 = .XAxis.Ori + BoxWidth * (j-1) + Incr/2
           x2 = x1 + GrpWidth + 1
           IF .YAxis.LogScale = TRUE THEN
            y1 = .YAxis.Ori
            y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSEIF .YAxis.Min >= 0 AND .YAxis.Max >= 0 THEN 'No Negative data
            y1 = .YAxis.Ori
            y2 = .YAxis.Ori - [VAL(.Data.Cell(i,j)) -.YAxis.Min] * .YAxis.ScaleFactor
           ELSE                                             'Have Negative values
            y1 = .YAxis.Zero
            IF VAL(.Data.Cell(i,j)) < 0 AND .YAxis.Max <> 0 THEN y1 = y1 + 1 'let 0 line show
            y2 = .YAxis.Zero - (VAL(.Data.Cell(i,j)) * .YAxis.ScaleFactor) + 1
           END IF
           Rectangle (.Buffer.Handle,x1,y1,x2,y2)
NoPlotHisto:
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
         NEXT i
        END WITH

       CASE csBar                                          'Grouped Bar Chart (spaces between bars)
        WITH QChart
         GrpWidth = (.XAxis.Len + .01) / .Data.RowCount
         BoxWidth = GrpWidth / (.Data.ColCount + .5)
         FOR i = 1 TO .Data.ColCount
          IF .Series(i).HatchStyle = hsSolid THEN
           .hBrush = CreateSolidBrush(.Series(i).Color)
          ELSE
           .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
          END IF
          .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
          FOR j = 1 TO .Data.RowCount
           IF .Data.Cell(i,j) = .Missing THEN GOTO NoPlotBar
           x1 = (.XAxis.Ori + BoxWidth / 4) + GrpWidth * (j - 1) + BoxWidth * (i - 1)
           x2 = x1 + BoxWidth + 1
           IF .YAxis.LogScale = TRUE THEN
            y1 = .YAxis.Ori
            y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSEIF .YAxis.Min >= 0 AND .YAxis.Max >= 0 THEN  'No Negative data
            y1 = .YAxis.Ori
            y2 = .YAxis.Ori - [VAL(.Data.Cell(i,j)) -.YAxis.Min] * .YAxis.ScaleFactor
           ELSE                                        'Have Negative values
            y1 = .YAxis.Zero
            IF VAL(.Data.Cell(i,j)) < 0 AND .YAxis.Max <> 0 THEN y1 = y1 + 1 'let 0 line show
            y2 = .YAxis.Zero - (VAL(.Data.Cell(i,j)) * .YAxis.ScaleFactor) + 1
           END IF
           Rectangle (.Buffer.Handle,x1,y1,x2,y2)
NoPlotBar:
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
         NEXT i
        END WITH

       CASE csStacked
        WITH QChart
         BoxWidth = (.XAxis.Len + .01) / (.Data.RowCount + 1)
         Incr = BoxWidth / .Data.RowCount
         FOR i = 1 TO .Data.RowCount
          FOR j = 1 TO .Data.ColCount
           IF .Series(j).HatchStyle = hsSolid THEN
            .hBrush = CreateSolidBrush(.Series(j).Color)
           ELSE
            .hBrush = CreateHatchBrush(.Series(j).HatchStyle, .Series(j).Color)
           END IF
           .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
           x1 = (.XAxis.Ori + Incr/2) + (BoxWidth + Incr) * (i-1)
           x2 = x1 + BoxWidth
           IF j = 1 THEN y1 = .YAxis.Ori ELSE y1 = y2
           y2 = y1 - (ABS(VAL(.Data.Cell(j,i)))- .YAxis.Min) * .YAxis.ScaleFactor
           Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2)
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
         NEXT i
        END WITH

       CASE csPctStacked
        WITH QChart
         BoxWidth = (.XAxis.Len + .01) / (.Data.RowCount +1)
         Incr = BoxWidth / .Data.RowCount
         FOR i = 1 TO .Data.RowCount
          Total = .RowTotal(i)
          IF Total = 0 THEN
           MESSAGEDLG("Division by Zero!", 1, 4, 0)
           .ClearAll
           EXIT SUB
          END IF
          FOR j = 1 TO .Data.ColCount
           IF .Series(j).HatchStyle = hsSolid THEN
            .hBrush = CreateSolidBrush(.Series(j).Color)
           ELSE
            .hBrush = CreateHatchBrush(.Series(j).HatchStyle, .Series(j).Color)
           END IF
           .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
           x1 = (.XAxis.Ori + Incr/2) + (BoxWidth + Incr) * (i-1)
           x2 = x1 + BoxWidth
           IF j = 1 THEN y1 = .YAxis.Ori ELSE y1 = y2
           y2 = y1 - (100 * (ABS(VAL(.Data.Cell(j,i)))/Total) * .YAxis.ScaleFactor)
           Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2)
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
         NEXT i
        END WITH

       END SELECT
       QChart.ChartExists = TRUE

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawXY (Overlay AS INTEGER)
  'X Points are (OriX + VAL * XScaleFactor - XMin) (ScaleFactor = Axis.Len / Axis.Max)
  'Y Points are (OriY - VAL * YScaleFactor - YMin) (i.e. ^^  = Pixels per Unit)

       DIM i  AS INTEGER                               'Loop counters...
       DIM j  AS INTEGER
       DIM x1 AS SINGLE                                'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE

       IF Overlay = FALSE THEN
        WITH QChart                                   'Set up the chart window
         .ClearBuffer                                'erase the current bitmap
         .SetSize
         .SetScale
         IF .Success = FALSE THEN                    'Log Scaling failed
          .ClearAll
          EXIT SUB
         END IF
         .SetSeries
         .DrawAxes
         .DrawTitles
         .DrawLabels
         .DrawLegend
        END WITH
       END IF

       SELECT CASE QChart.ChartStyle

       CASE csPoints
        WITH QChart
         FOR i = 2 TO .Data.ColCount STEP 2
          FOR j = 1 TO .Data.RowCount

           IF .Data.Cell(i-1,j)=.Missing THEN GOTO NoPlotXYPt   'If data missing
           IF .Data.Cell(i,j)=.Missing THEN GOTO NoPlotXYPt

           IF .XAxis.LogScale = FALSE THEN
            x1 = .XAxis.Ori + [VAL(.Data.Cell(i-1,j))-.XAxis.Min] * .XAxis.ScaleFactor
           ELSE
            x1 = .XAxis.Ori + (.Log10(VAL(.Data.Cell(i-1,j)))-.XAxis.Min) * _
             .XAxis.ScaleFactor
           END IF

           IF .YAxis.LogScale = FALSE THEN
            y1 = .YAxis.Ori - [VAL(.Data.Cell(i,j))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSE
            y1 = .YAxis.Ori - (.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min) * _
             .YAxis.ScaleFactor
           END IF
           .DrawMarker (x1, y1, .Series(i\2).Marker, .Series(i\2).Color)
NoPlotXYPt:
          NEXT j
         NEXT i
        END WITH

       CASE csLines
        WITH QChart
         SetBkColor (.Buffer.Handle, .PlotAreaColor)
         FOR i = 2 TO .Data.ColCount STEP 2
          .hPen = CreatePen(.Series(i\2).LineStyle, .Series(i\2).LineWidth, .Series(i\2).Color)
          .OldPen = SelectObject(.Buffer.Handle, .hPen)
          FOR j = 1 TO .Data.RowCount - 1

           IF .Data.Cell(i-1,j)=.Missing OR .Data.Cell(i-1,j+1)=.Missing _'If data missing
            THEN GOTO NoPlotXYLn
           IF .Data.Cell(i,j)=.Missing OR .Data.Cell(i,j+1)=.Missing _
            THEN GOTO NoPlotXYLn

           IF .XAxis.LogScale = FALSE THEN
            x1 = [VAL(.Data.Cell(i-1,j))-.XAxis.Min] * .XAxis.ScaleFactor
            x2 = [VAL(.Data.Cell(i-1,j+1))-.XAxis.Min] * .XAxis.ScaleFactor
           ELSE
            x1 = [.Log10(VAL(.Data.Cell(i-1,j)))-.XAxis.Min] * .XAxis.ScaleFactor
            x2 = [.Log10(VAL(.Data.Cell(i-1,j+1)))-.XAxis.Min] * .XAxis.ScaleFactor
           END IF

           IF .YAxis.LogScale = FALSE THEN
            y1 = [VAL(.Data.Cell(i,j))-.YAxis.Min] * .YAxis.ScaleFactor
            y2 = [VAL(.Data.Cell(i,j+1))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSE
            y1 = [.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min] *.YAxis.ScaleFactor
            y2 = [.Log10(VAL(.Data.Cell(i,j+1)))-.YAxis.Min] *.YAxis.ScaleFactor
           END IF

           x1 = .XAxis.Ori + x1
           y1 = .YAxis.Ori - y1
           x2 = .XAxis.Ori + x2
           y2 = .YAxis.Ori - y2

              '--- Use API to draw lines
           MoveToEx(.Buffer.handle,x1,y1,0)
           LineTo(.Buffer.handle,x2,y2)
NoPlotXYLn:
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldPen))
         NEXT i
        END WITH

       CASE csBoth
        WITH QChart
         .ChartStyle = csPoints                    'Draw Points
         .DrawXY (FALSE)                           'Draw Points (erase current)
         .ChartStyle = csLines                     'AND Lines
         .DrawXY (TRUE)                            'Overlay Lines (do NOT erase)
         .ChartStyle = csBoth                      'Reset ChartStyle for repaints
        END WITH

       END SELECT
       QChart.ChartExists = TRUE

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawLine (Overlay AS INTEGER)
  'X Points are (OriX + VAL * XScaleFactor - XMin) (ScaleFactor = Axis.Len / Axis.Max)
  'Y Points are (OriY - VAL * YScaleFactor - YMin) (i.e. ^^  = Pixels per Unit)

       DIM i  AS INTEGER                               'Loop counters...
       DIM j  AS INTEGER
       DIM x1 AS SINGLE                                'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE

       IF Overlay = FALSE THEN
        WITH QChart                                   'Set up the chart window
         .ClearBuffer                                'erase the current bitmap
         .SetSize
         .SetScale
         IF .Success = FALSE THEN                    'Log Scaling failed
          .ClearAll
          EXIT SUB
         END IF
         .SetSeries
         .XAxis.Div = .Data.RowCount - 1             'Forces a grid line for each X Label
         .XAxis.DrawZero = FALSE                     'No Values on the X
         .DrawAxes                                   'in DrawAxes Sub
         .DrawTitles
         .DrawLabels
         .DrawLegend
        END WITH
       END IF

       SELECT CASE QChart.ChartStyle

       CASE csPoints
        WITH QChart
         FOR i = 1 TO .Data.ColCount
          FOR j = 1 TO .Data.RowCount

           IF .Data.Cell(i,j)=.Missing THEN GOTO NoPlotLinePt 'If data missing

           x1 = .XAxis.Ori + (j-1) * .Xaxis.TicIntvl
           IF .YAxis.LogScale = FALSE THEN
            y1 = .YAxis.Ori - [VAL(.Data.Cell(i,j))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSE
            y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min] *.YAxis.ScaleFactor
           END IF
           .DrawMarker (x1, y1, .Series(i).Marker, .Series(i).Color)
NoPlotLinePt:
          NEXT j
         NEXT i
        END WITH

       CASE csLines
        WITH QChart
         SetBkColor (.Buffer.Handle, .PlotAreaColor)
         FOR i = 1 TO .Data.ColCount
          .hPen = CreatePen(.Series(i).LineStyle, .Series(i).LineWidth, .Series(i).Color)
          .OldPen = SelectObject(.Buffer.Handle, .hPen)
          FOR j = 1 TO .Data.RowCount - 1

           IF .Data.Cell(i,j)=.Missing OR .Data.Cell(i,j+1)=.Missing _ 'If data missing
            THEN GOTO NoPlotLineLn

           x1 = .XAxis.Ori + (j-1) * .Xaxis.TicIntvl
           x2 = .XAxis.Ori + (j) * .Xaxis.TicIntvl
           IF .YAxis.LogScale = FALSE THEN
            y1 = .YAxis.Ori - [VAL(.Data.Cell(i,j))-.YAxis.Min] * .YAxis.ScaleFactor
            y2 = .YAxis.Ori - [VAL(.Data.Cell(i,j+1))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSE
            y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,j)))-.YAxis.Min] *.YAxis.ScaleFactor
            y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,j+1)))-.YAxis.Min] *.YAxis.ScaleFactor
           END IF

              '--- Use API to draw lines
           MoveToEx(.Buffer.handle,x1,y1,0)
           LineTo(.Buffer.handle,x2,y2)
NoPlotLineLn:
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldPen))
         NEXT i

        END WITH

       CASE csBoth
        WITH QChart
         .ChartStyle = csPoints                    'Draw Points
         .DrawLine (FALSE)                         'Draw Points (erase current)
         .ChartStyle = csLines                     'AND Lines
         .DrawLine (TRUE)                          'Overlay Lines (do NOT erase)
         .ChartStyle = csBoth                      'Reset ChartStyle for repaints
        END WITH

       END SELECT
       QChart.ChartExists = TRUE

      END SUB
'---------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawBox (Overlay AS INTEGER)

  'X Points are (OriX + VAL * XScaleFactor - XMin) (ScaleFactor = Axis.Len / Axis.Max)
  'Y Points are (OriY - VAL * YScaleFactor - YMin) (i.e. ^^  = Pixels per Unit)

       DIM i  AS INTEGER                               'Loop counters...
       DIM j  AS INTEGER
       DIM x1 AS SINGLE                                'for readability
       DIM y1 AS SINGLE
       DIM x2 AS SINGLE
       DIM y2 AS SINGLE
	dim open1 as single
	dim high1 as single
	dim low1 as single
	dim close1 as single
	dim volume1 as single
	dim open2 as single
	dim high2 as single
	dim low2 as single
	dim close2 as single
	dim volume2 as single
       DIM XIntvl AS SINGLE                            'Distance between plots
       DIM LinWidth AS LONG                            'Width od Mid Line
       DIM LinColor AS LONG                            'Change for Hi-Lo
       DIM BoxWidth AS LONG                            'Width of bars

       IF Overlay = FALSE THEN
        WITH QChart                                   'Set up the chart window
         .ClearBuffer                                'erase the current bitmap

         .SetSize
         .SetScale
         IF .Success = FALSE THEN                    'Log Scaling failed
          .ClearAll
          EXIT SUB
         END IF
         .SetSeries
         .Xaxis.Div = .Data.ColCount
         .XAxis.DrawZero = FALSE                     'No Values on the X
         .DrawAxes
         .DrawTitles
         .DrawLabels
         .DrawLegend
        END WITH
       END IF

	WITH QChart

        XIntvl = .XAxis.Len / .Data.ColCount
        LinWidth = XIntvl * .75
        BoxWidth = LinWidth * .75

	select case charttypecomboindex

	case 0:
        FOR i = 1 TO .Data.ColCount

        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = x1
         IF .YAxis.LogScale = FALSE THEN
          y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max	  
	ELSE
          y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
          y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         END IF
if i=clickedbarnb then
.Buffer.Line (x1 , y1 , x2 , y2 , green)

else
         .Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
end if
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         x1 = x1 - BoxWidth / 2
         x2 = x1 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
	y1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3	  
         ELSE
          y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
         Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
         'IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
         'IF .GreyScale = TRUE THEN LinColor = .Colors(21)
         'IF .Series(i).HatchStyle = hsSolid THEN
         ' .hBrush = CreateSolidBrush(LinColor)
         'ELSE
         ' .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
         'END IF
         '.OldBrush = SelectObject(.Buffer.Handle, .hBrush)
         'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
         'DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i	

	case 1:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          y1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	if i<.Data.ColCount then
	.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
	end if
         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 2:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         'x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          y1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	.Buffer.pset (x1 , y1 , .fgColor)
	.Buffer.pset (x1-1 , y1 , .fgColor)
	.Buffer.pset (x1-1 , y1+1 , .fgColor)
	.Buffer.pset (x1 , y1+1 , .fgColor)
	.Buffer.pset (x1+1 , y1+1 , .fgColor)
	.Buffer.pset (x1+1 , y1 , .fgColor)
	.Buffer.pset (x1+1 , y1-1 , .fgColor)
	.Buffer.pset (x1 , y1-1 , .fgColor)
	.Buffer.pset (x1-1 , y1-1 , .fgColor)
         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 3:
        FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = x1
         IF .YAxis.LogScale = FALSE THEN
          y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         ELSE
          y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
          y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         END IF
         .Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         x1 = x1 - BoxWidth / 1.75
         x2 = x1 + BoxWidth / 1.75
         IF .YAxis.LogScale = FALSE THEN
          y1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         ELSE
          y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
	.buffer.line(x1,y1,x2,y1, .fgColor)
	.buffer.line(x2,y2,x2+BoxWidth / 1.75,y2, .fgColor)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
         'IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
         'IF .GreyScale = TRUE THEN LinColor = .Colors(21)
         'IF .Series(i).HatchStyle = hsSolid THEN
         ' .hBrush = CreateSolidBrush(LinColor)
         'ELSE
         ' .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
         'END IF
         '.OldBrush = SelectObject(.Buffer.Handle, .hBrush)
         'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
         'DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 4:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          high1 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          high2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,1))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  low1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          low2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,5))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  open1 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          open2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  close1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          close2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	if i<.Data.ColCount then
	.Buffer.Line (x1 , high1 , x2 , high2 , .fgColor)
	.Buffer.Line (x1 , low1 , x2 , low2 , .fgColor)
	.Buffer.Line (x1 , open1 , x2 , open2 , .colors(3))
	.Buffer.Line (x1 , close1 , x2 , close2 , .colors(2))
	end if
         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 5:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          high1 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'high2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,1))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  low1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'low2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,5))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  open1 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'open2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  close1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'close2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	
	.Buffer.Line (x1 , low1 , x1 , high1 , .fgColor)
	.Buffer.Line (x1 - BoxWidth , close1 , x1 + BoxWidth , open1 , .fgColor)
	if open1<=close1 then
	.Buffer.Line (x1 - BoxWidth , close1 , x1 , low1 , .fgColor)
	.Buffer.Line (x1 + BoxWidth , open1 , x1 , high1 , .fgColor)	
	end if	
	if open1>close1 then
	.Buffer.Line (x1 - BoxWidth , close1 , x1 , high1 , .fgColor)
	.Buffer.Line (x1 + BoxWidth , open1 , x1 , low1 , .fgColor)	
	end if

         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 6:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          'high1 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'high2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,1))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  'low1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'low2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,5))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  'open1 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'open2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  close1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          close2 = .YAxis.Ori - [VAL(.Data.Cell(i-1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	

	.Buffer.Line (x1 - BoxWidth , close1 , x1 + BoxWidth , close1 , .fgColor)
	
	.Buffer.Line (x1 - BoxWidth , close1 , x1 - BoxWidth , close2 , .fgColor)
	

         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

	case 7:
	FOR i = 1 TO .Data.ColCount
        '--- Draw Min - Max Line (Whiskers)
         x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2
         x2 = .XAxis.Ori + XIntvl * (i+1-1) + XIntvl / 2
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,5)))-.YAxis.Min] * .YAxis.ScaleFactor 'Min
         ' y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,1)))-.YAxis.Min] * .YAxis.ScaleFactor 'Max
         'END IF
         '.Buffer.Line (x1 , y1 , x2 , y2 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y1 , x2 + LinWidth / 4 , y1 , .fgColor)
         '.Buffer.Line (x1 - LinWidth / 4 , y2 , x2 + LinWidth / 4 , y2 , .fgColor)

        '--- Draw Quartile Box
         'x1 = x1 - BoxWidth / 2
         'x2 = x2 + BoxWidth + 1
         IF .YAxis.LogScale = FALSE THEN
          'high1 = .YAxis.Ori - [VAL(.Data.Cell(i,1))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'high2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,1))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  'low1 = .YAxis.Ori - [VAL(.Data.Cell(i,5))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'low2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,5))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  'open1 = .YAxis.Ori - [VAL(.Data.Cell(i,2))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'open2 = .YAxis.Ori - [VAL(.Data.Cell(i+1,2))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
	  'close1 = .YAxis.Ori - [VAL(.Data.Cell(i,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'close2 = .YAxis.Ori - [VAL(.Data.Cell(i-1,4))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         'ELSE
          'y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,4)))-.YAxis.Min] * .YAxis.ScaleFactor + 1 'Q1
          'y2 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,2)))-.YAxis.Min] * .YAxis.ScaleFactor     'Q3
         END IF
         IF .Series(i).HatchStyle = hsSolid THEN
          .hBrush = CreateSolidBrush(.Series(i).Color)
         ELSE
          .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
         END IF
         .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
	

	'.Buffer.Line (x1 - BoxWidth , close1 , x1 + BoxWidth , close1 , .fgColor)
	
	'.Buffer.Line (x1 - BoxWidth , close1 , x1 - BoxWidth , close2 , .fgColor)
	

         'Rectangle (.Buffer.Handle,x1,y1,x2+1,y2)
         DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

        '--- Draw Median Line
         'x1 = .XAxis.Ori + XIntvl * (i-1) + XIntvl / 2 - LinWidth / 2
         'x2 = x1 + LinWidth + 1
         'IF .YAxis.LogScale = FALSE THEN
         ' y1 = .YAxis.Ori - [VAL(.Data.Cell(i,3))-.YAxis.Min] * .YAxis.ScaleFactor          'Q2
         'ELSE
         ' y1 = .YAxis.Ori - [.Log10(VAL(.Data.Cell(i,3)))-.YAxis.Min] * .YAxis.ScaleFactor  'Q2
         'END IF
         'y2 = y1
        ' IF .ChartStyle = csHiLo THEN LinColor = .Series(i).Color ELSE LinColor = .Colors(12)
        ' IF .GreyScale = TRUE THEN LinColor = .Colors(21)
        ' IF .Series(i).HatchStyle = hsSolid THEN
        '  .hBrush = CreateSolidBrush(LinColor)
        ' ELSE
        '  .hBrush = CreateHatchBrush(.Series(i).HatchStyle, LinColor)
        ' END IF
       '  .OldBrush = SelectObject(.Buffer.Handle, .hBrush)
        ' 'Rectangle (.Buffer.Handle,x1,y1+1,x2+1,y2-2)
        ' DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))
        NEXT i

dim angle as double,cs as double,sn as double,c as double
c=graphvspace*0.5
'dim i as integer
dim ratio as double
dim x7 as integer
dim y7 as integer
dim closexarray(1 to 1000) as double
dim closeyarray(1 to 1000) as double

i=1

x7=graphhbegin+graphhspace*0.5
y7=graphvbegin+graphvspace*0.5

for angle=0 to 6.28 step 6.28/.Data.ColCount

if (angle>=1.57 and angle<3.14) then
'close1 = VAL(.Data.Cell(i-.Data.ColCount/4-1,4))
'ratio=(close1-graphvlow)/graphpricespace
close1 = .YAxis.Ori - [VAL(.Data.Cell(i-.Data.ColCount/4-1,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1
ratio=(graphvspace-(close1-graphvbegin))/graphvspace
cs=c*ratio*cos(angle)
sn=c*ratio*sin(angle)
'.buffer.line(x7,y7,x7-cs,y7-sn,pencolor)
closexarray(i)=x7-cs
closeyarray(i)=y7-sn
if i-.Data.ColCount/4-1>1 then
.buffer.line(closexarray(i),closeyarray(i),closexarray(i-1),closeyarray(i-1),pencolor)
end if
end if

i++
next angle

i=1

for angle=6.28 to 0 step -6.28/.Data.ColCount

if (angle>=1.57 and angle<3.14) then
'close1 = VAL(.Data.Cell(i-.Data.ColCount/4-1,4))
'ratio=(close1-graphvlow)/graphpricespace
close1 = .YAxis.Ori - [VAL(.Data.Cell(i-.Data.ColCount/4-1,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1
ratio=(graphvspace-(close1-graphvbegin))/graphvspace
cs=c*ratio*cos(angle)
sn=c*ratio*sin(angle)
'.buffer.line(x7,y7,x7-cs,y7+sn,pencolor)
closexarray(i)=x7-cs
closeyarray(i)=y7+sn
.buffer.line(closexarray(i),closeyarray(i),closexarray(i-1),closeyarray(i-1),pencolor)
end if

i++
next angle

i=1

for angle=6.28 to 0 step -6.28/.Data.ColCount

if (angle>=0 and angle<1.57) then
'close1 = VAL(.Data.Cell(i-.Data.ColCount/4-1,4))
'ratio=(close1-graphvlow)/graphpricespace
close1 = .YAxis.Ori - [VAL(.Data.Cell(i-.Data.ColCount/4-1,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1
ratio=(graphvspace-(close1-graphvbegin))/graphvspace
cs=c*ratio*cos(angle)
sn=c*ratio*sin(angle)
'.buffer.line(x7,y7,x7-cs,y7+sn,pencolor)
closexarray(i)=x7-cs
closeyarray(i)=y7+sn
.buffer.line(closexarray(i),closeyarray(i),closexarray(i-1),closeyarray(i-1),pencolor)
end if

i++
next angle

i=1

for angle=0 to 6.28 step 6.28/.Data.ColCount

if (angle>=0 and angle<1.57) then
'close1 = VAL(.Data.Cell(i+.Data.ColCount*0.75-1,4))
'ratio=(close1-graphvlow)/graphpricespace
close1 = .YAxis.Ori - [VAL(.Data.Cell(i+.Data.ColCount*0.75-1,4))-.YAxis.Min] * .YAxis.ScaleFactor + 1
ratio=(graphvspace-(close1-graphvbegin))/graphvspace
cs=c*ratio*cos(angle)
sn=c*ratio*sin(angle)
'.buffer.line(x7,y7,x7-cs,y7-sn,pencolor)
closexarray(i)=x7-cs
closeyarray(i)=y7-sn
if i+.Data.ColCount*0.75-1=.Data.ColCount*0.75 then
.buffer.line(closexarray(i),closeyarray(i),closexarray(.Data.ColCount),closeyarray(.Data.ColCount),pencolor)
end if
if i+.Data.ColCount*0.75-1>.Data.ColCount*0.75 then
.buffer.line(closexarray(i),closeyarray(i),closexarray(i-1),closeyarray(i-1),pencolor)
end if
end if

i++
next angle

cs=c*cos(angle)
sn=c*sin(angle)
for angle=0 to 1.57 step 0.001
'if cos(angle)<>0 and sin(angle)<>0 then
cs=c*cos(angle)
sn=c*sin(angle)
.buffer.pset(x7+cs,y7+sn,pencolor)
.buffer.pset(x7-cs,y7+sn,pencolor)
.buffer.pset(x7+cs,y7-sn,pencolor)
.buffer.pset(x7-cs,y7-sn,pencolor)
'end if
next angle

.buffer.pset(x7,y7,pencolor)
.buffer.pset(x7-1,y7,pencolor)
.buffer.pset(x7-1,y7+1,pencolor)
.buffer.pset(x7,y7+1,pencolor)
.buffer.pset(x7+1,y7+1,pencolor)
.buffer.pset(x7+1,y7,pencolor)
.buffer.pset(x7+1,y7-1,pencolor)
.buffer.pset(x7,y7-1,pencolor)
.buffer.pset(x7-1,y7-1,pencolor)

	end select

       END WITH
       QChart.ChartExists = TRUE       

' DRAWING INDICATORS ON CHART WINDOW
if useindi.checked=1 then
WITH QChart

.XAxis.ScaleFactor=.XAxis.Len / .XAxis.Div

         SetBkColor (.Buffer.Handle, .PlotAreaColor)

         FOR i = 2 TO .DATA_INDI.ColCount STEP 2
          .hPen = CreatePen(.Series(i\2).LineStyle, .Series(i\2).LineWidth, indicolor(i))'.Series(i\2).Color)
          .OldPen = SelectObject(.Buffer.Handle, .hPen)
          FOR j = 1 TO .DATA_INDI.RowCount - 1

           IF .DATA_INDI.Cell(i-1,j)=.Missing OR .DATA_INDI.Cell(i-1,j+1)=.Missing _'If data missing
            THEN GOTO NoPlotXYLn
           IF .DATA_INDI.Cell(i,j)=.Missing OR .DATA_INDI.Cell(i,j+1)=.Missing _
            THEN GOTO NoPlotXYLn
           IF .XAxis.LogScale = FALSE THEN
            x1 = [VAL(.DATA_INDI.Cell(i-1,j))-.XAxis.Min] * .XAxis.ScaleFactor
            x2 = [VAL(.DATA_INDI.Cell(i-1,j+1))-.XAxis.Min] * .XAxis.ScaleFactor
           ELSE
            x1 = [.Log10(VAL(.DATA_INDI.Cell(i-1,j)))-.XAxis.Min] * .XAxis.ScaleFactor
            x2 = [.Log10(VAL(.DATA_INDI.Cell(i-1,j+1)))-.XAxis.Min] * .XAxis.ScaleFactor
           END IF

           IF .YAxis.LogScale = FALSE THEN
            y1 = [VAL(.DATA_INDI.Cell(i,j))-.YAxis.Min] * .YAxis.ScaleFactor
            y2 = [VAL(.DATA_INDI.Cell(i,j+1))-.YAxis.Min] * .YAxis.ScaleFactor
           ELSE
            y1 = [.Log10(VAL(.DATA_INDI.Cell(i,j)))-.YAxis.Min] *.YAxis.ScaleFactor
            y2 = [.Log10(VAL(.DATA_INDI.Cell(i,j+1)))-.YAxis.Min] *.YAxis.ScaleFactor
           END IF

           x1 = .XAxis.Ori + x1
           y1 = .YAxis.Ori - y1
           x2 = .XAxis.Ori + x2
           y2 = .YAxis.Ori - y2

              '--- Use API to draw lines
           'MoveToEx(.Buffer.handle,x1,y1,0)
           'LineTo(.Buffer.handle,x2,y2)
	   .Buffer.line(x1,y1,x2,y2,indicolor(i))

NoPlotXYLn:
          NEXT j
          DeleteObject(SelectObject(.Buffer.Handle, .OldPen))

         NEXT i

        END WITH
end if
      END SUB

private:
sub drawtrendline(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawfibofan(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
'.restorebuffertmp
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
dim x4 as double
x4=x3+(x3-x2)
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x4,y3)
.Buffer.line(x2,y2,x4,y3,pencolor)
.Buffer.textout(x4-40,y3,"50",pencolor,&hffffff)
x4=x3+(x3-x2)*0.618
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x4,y3)
.Buffer.line(x2,y2,x4,y3,pencolor)
.Buffer.textout(x4-40,y3,"38.2",pencolor,&hffffff)
x4=x3+(x3-x2)*1.618
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x4,y3)
.Buffer.line(x2,y2,x4,y3,pencolor)
.Buffer.textout(x4-40,y3,"61.8",pencolor,&hffffff)
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawfiboret(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
.Buffer.textout(x2-30,y2,"100",pencolor,&hffffff)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y2)
'MoveToEx(.Buffer.handle,x2,y3,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y2,pencolor)
.Buffer.line(x2,y3,x3,y3,pencolor)
.Buffer.textout(x2-30,y3,"0",pencolor,&hffffff)
dim y4 as double
y4=y3-(y3-y2)*0.236
'MoveToEx(.Buffer.handle,x2,y4,0)
'LineTo(.Buffer.handle,x3,y4)
.Buffer.line(x2,y4,x3,y4,pencolor)
.Buffer.textout(x2-30,y4,"23.6",pencolor,&hffffff)
y4=y3-(y3-y2)*0.382
'MoveToEx(.Buffer.handle,x2,y4,0)
'LineTo(.Buffer.handle,x3,y4)
.Buffer.line(x2,y4,x3,y4,pencolor)
.Buffer.textout(x2-30,y4,"38.2",pencolor,&hffffff)
y4=y3-(y3-y2)*0.5
'MoveToEx(.Buffer.handle,x2,y4,0)
'LineTo(.Buffer.handle,x3,y4)
.Buffer.line(x2,y4,x3,y4,pencolor)
.Buffer.textout(x2-30,y4,"50",pencolor,&hffffff)
y4=y3-(y3-y2)*0.618
'MoveToEx(.Buffer.handle,x2,y4,0)
'LineTo(.Buffer.handle,x3,y4)
.Buffer.line(x2,y4,x3,y4,pencolor)
.Buffer.textout(x2-30,y4,"61.8",pencolor,&hffffff)
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawpara(xb,yb,xc,yc,xd,yd)
        
    defsng x2,y2,x3,y3,x4,y4   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
x4=xd
y4=yd
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz2.bmp
'MoveToEx(.Buffer.handle,x3-1,y3,0)
'LineTo(.Buffer.handle,x3+x4-x2,y3+y4-y2)
.Buffer.line(x3,y3,x3+x4-x2,y3+y4-y2,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawpitchfork(xb,yb,xc,yc,xd,yd)
        
    defsng x2,y2,x3,y3,x4,y4   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
x4=xd
y4=yd
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz2.bmp
'MoveToEx(.Buffer.handle,x3-1,y3,0)
'LineTo(.Buffer.handle,x3+x4-x2,y3+y4-y2)
dim medx as integer,medy as integer
medx=x4+(x3-x4)*0.5
medy=y4+(y3-y4)*0.5
.Buffer.line(x2,y2,medx+(medx-x2)*100,medy+(medy-y2)*100,pencolor)
.Buffer.line(x4,y4,x4+(medx-x2)*100,y4+(medy-y2)*100,pencolor)
.Buffer.line(x3,y3,x3+(medx-x2)*100,y3+(medy-y2)*100,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawpriceext(xb,yb,xc,yc,xd,yd)
        
    defsng x2,y2,x3,y3,x4,y4   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
x4=xd
y4=yd
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz2.bmp
'MoveToEx(.Buffer.handle,x3-1,y3,0)
'LineTo(.Buffer.handle,x3+x4-x2,y3+y4-y2)
dim quarter as double
quarter=(y4-y2)/4
.Buffer.line(x3,y3,x3+500,y3,pencolor)
.Buffer.textout(x3-40,y3,"0%",pencolor,&hffffff)
.Buffer.line(x3,y3+quarter,x3+500,y3+quarter,pencolor)
.Buffer.textout(x3-40,y3+quarter,"25%",pencolor,&hffffff)
.Buffer.line(x3,y3+quarter*2,x3+500,y3+quarter*2,pencolor)
.Buffer.textout(x3-40,y3+quarter*2,"50%",pencolor,&hffffff)
.Buffer.line(x3,y3+quarter*3,x3+500,y3+quarter*3,pencolor)
.Buffer.textout(x3-40,y3+quarter*3,"75%",pencolor,&hffffff)
.Buffer.line(x3,y3+quarter*4,x3+500,y3+quarter*4,pencolor)
.Buffer.textout(x3-40,y3+quarter*4,"100%",pencolor,&hffffff)

.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawtimeext(xb,yb,xc,yc,xd,yd)
        
    defsng x2,y2,x3,y3,x4,y4   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
x4=xd
y4=yd
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz2.bmp
'MoveToEx(.Buffer.handle,x3-1,y3,0)
'LineTo(.Buffer.handle,x3+x4-x2,y3+y4-y2)
dim quarter as double
quarter=(x4-x2)/4
.Buffer.line(x3,y3,x3,y3+500,pencolor)
.Buffer.textout(x3,y3-40,"0%",pencolor,&hffffff)
.Buffer.line(x3+quarter,y3,x3+quarter,y3+500,pencolor)
.Buffer.textout(x3+quarter,y3-40,"25%",pencolor,&hffffff)
.Buffer.line(x3+quarter*2,y3,x3+quarter*2,y3+500,pencolor)
.Buffer.textout(x3+quarter*2,y3-40,"50%",pencolor,&hffffff)
.Buffer.line(x3+quarter*3,y3,x3+quarter*3,y3+500,pencolor)
.Buffer.textout(x3+quarter*3,y3-40,"75%",pencolor,&hffffff)
.Buffer.line(x3+quarter*4,y3,x3+quarter*4,y3+500,pencolor)
.Buffer.textout(x3+quarter*4,y3-40,"100%",pencolor,&hffffff)

.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawhline(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y2)
.Buffer.line(x2,y2,x3,y2,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawvline(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x2,y3)
.Buffer.line(x2,y2,x2,y3,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawsqr(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
dim x4 as double,y4 as double
if y3>y2 then
ang_a=ang_a*-1
end if
x4=cos((ang_a-90)*rad)*c+x3
y4=y3-sin((ang_a-90)*rad)*c
'MoveToEx(.Buffer.handle,x3,y3,0)
'LineTo(.Buffer.handle,x4,y4)
.Buffer.line(x3,y3,x4,y4,pencolor)

ang_a=ang_a-90
'MoveToEx(.Buffer.handle,x4,y4,0)
.Buffer.line(x4,y4,cos((ang_a-90)*rad)*c+x4,y4-sin((ang_a-90)*rad)*c,pencolor)
x4=cos((ang_a-90)*rad)*c+x4
y4=y4-sin((ang_a-90)*rad)*c
'LineTo(.Buffer.handle,x4,y4)

ang_a=ang_a-90
.Buffer.line(x4,y4,cos((ang_a-90)*rad)*c+x4,y4-sin((ang_a-90)*rad)*c,pencolor)
'MoveToEx(.Buffer.handle,x4,y4,0)
'x4=cos((ang_a-90)*rad)*c+x4
'y4=y4-sin((ang_a-90)*rad)*c
'LineTo(.Buffer.handle,x4,y4)
.buffertmp.bmp=.buffer.bmp
.paintchart

end with

end sub

private:
sub drawtri(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
dim x4 as double,y4 as double
if y3>y2 then
ang_a=ang_a*-1
end if
x4=cos((ang_a-120)*rad)*c+x3
y4=y3-sin((ang_a-120)*rad)*c
'MoveToEx(.Buffer.handle,x3,y3,0)
'LineTo(.Buffer.handle,x4,y4)
.Buffer.line(x3,y3,x4,y4,pencolor)

ang_a=ang_a-120
'MoveToEx(.Buffer.handle,x4,y4,0)
.Buffer.line(x4,y4,cos((ang_a-120)*rad)*c+x4,y4-sin((ang_a-120)*rad)*c,pencolor)
'x4=cos((ang_a-120)*rad)*c+x4
'y4=y4-sin((ang_a-120)*rad)*c
'LineTo(.Buffer.handle,x4,y4)
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawcircle(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim angle as double,cs as double,sn as double
for angle=0 to 1.57 step 0.001
if cos(angle)<>0 and sin(angle)<>0 then
cs=c*cos(angle)
sn=c*sin(angle)
.buffer.pset(x2+cs,y2+sn,pencolor)
.buffer.pset(x2-cs,y2+sn,pencolor)
.buffer.pset(x2+cs,y2-sn,pencolor)
.buffer.pset(x2-cs,y2-sn,pencolor)
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawcross(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.line(x2,y2,x3,y3,pencolor)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
dim x4 as double,y4 as double
if y3>y2 then
ang_a=ang_a*-1
end if
'MoveToEx(.Buffer.handle,x3-(x3-x2)/2,y3-(y3-y2)/2,0)
'LineTo(.Buffer.handle,x4,y4)
dim ilcross as integer
for ilcross=0 to crosslevelslist.itemcount-1
x4=cos((ang_a-90)*rad)*c*0.5+x3-(x3-x2)*val(crosslevelslist.item(ilcross))
y4=y3-(y3-y2)*val(crosslevelslist.item(ilcross))-sin((ang_a-90)*rad)*c*0.5
.Buffer.line(x3-(x3-x2)*val(crosslevelslist.item(ilcross)),y3-(y3-y2)*val(crosslevelslist.item(ilcross)),x4,y4,pencolor)
next ilcross
'MoveToEx(.Buffer.handle,x3-(x3-x2)/2,y3-(y3-y2)/2,0)
'LineTo(.Buffer.handle,x4,y4)
for ilcross=0 to crosslevelslist.itemcount-1
x4=cos((ang_a+90)*rad)*c*0.5+x3-(x3-x2)*val(crosslevelslist.item(ilcross))
y4=y3-(y3-y2)*val(crosslevelslist.item(ilcross))-sin((ang_a+90)*rad)*c*0.5
.Buffer.line(x3-(x3-x2)*val(crosslevelslist.item(ilcross)),y3-(y3-y2)*val(crosslevelslist.item(ilcross)),x4,y4,pencolor)
next ilcross
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawinvcircle(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim angle as double,cs as double,sn as double
for angle=0 to 1.57 step 0.001
if cos(angle)<>0 and sin(angle)<>0 then
cs=c/cos(angle)
sn=c/sin(angle)
.buffer.pset(x2+cs,y2+sn,pencolor)
.buffer.pset(x2-cs,y2+sn,pencolor)
.buffer.pset(x2+cs,y2-sn,pencolor)
.buffer.pset(x2-cs,y2-sn,pencolor)
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawaiming(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x2,y3)
graphcursorpos2
.Buffer.line(x3,0,x3,graph.height,pencolor)
.Buffer.textout(x3,graphvspace+45,graphdateoncur,pencolor,&hffffff)
.Buffer.line(0,y3,graph.width,y3,pencolor)
.Buffer.textout(0,y3,str$(graphpriceoncur),pencolor,&hffffff)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawhand(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
'.ClearBuffer
'.restorebuffertmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x3,y3)
.Buffer.pset(x3,y3,pencolor)
.Buffer.pset(x3-1,y3,pencolor)
.Buffer.pset(x3-1,y3-1,pencolor)
.Buffer.pset(x3,y3-1,pencolor)
.Buffer.pset(x3+1,y3-1,pencolor)
.Buffer.pset(x3+1,y3,pencolor)
.Buffer.pset(x3+1,y3+1,pencolor)
.Buffer.pset(x3,y3+1,pencolor)
.Buffer.pset(x3-1,y3+1,pencolor)
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawsin(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim angle as double,cs as double,sn as double
for angle=0 to 50.24 step 0.005
if sin(angle<>0) then
.buffer.pset(x2+angle*a,y2+b*sin(angle),pencolor)
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawlog(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim ilog as double,cs as double,sn as double
for ilog=1 to 1000 step 0.1
if log(ilog)<>0 then
if reverse=0 then
if flip=0 then
.buffer.pset(x3+ilog,y3-log(ilog)*logamplitude,pencolor)
else
.buffer.pset(x3+ilog-1001,y3-log(1001-ilog)*logamplitude,pencolor)
end if
else
if flip=0 then
.buffer.pset(x3+ilog,y3+log(ilog)*logamplitude,pencolor)
else
.buffer.pset(x3+ilog-1001,y3+log(1001-ilog)*logamplitude,pencolor)
end if
end if
'for ilog=0 to 21 step 0.003
'if exp(ilog)<>0 then
'.buffer.pset(x3+ilog*logamplitude,y3-exp(ilog),pencolor)
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawexp(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim ilog as double,cs as double,sn as double
'for ilog=1 to 1000 step 0.1
'if log(ilog)<>0 then
'.buffer.pset(x3+ilog,y3-log(ilog)*logamplitude,pencolor)
for ilog=0 to 21 step 0.003
if exp(ilog)<>0 then
if reverse=0 then
if flip=0 then
.buffer.pset(x3+ilog*logamplitude,y3-exp(ilog),pencolor)
else
.buffer.pset(x3+(ilog-21)*logamplitude,y3-exp(21-ilog),pencolor)
end if
else
if flip=0 then
.buffer.pset(x3+ilog*logamplitude,y3+exp(ilog),pencolor)
else
.buffer.pset(x3+(ilog-21)*logamplitude,y3+exp(21-ilog),pencolor)
end if
end if
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawellipse(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim angle as double,cs as double,sn as double
for angle=0 to 1.57 step 0.001
if cos(angle)<>0 and sin(angle)<>0 then
cs=a*cos(angle)
sn=b*sin(angle)
.buffer.pset(x3+cs,y2+sn,pencolor)
.buffer.pset(x3-cs,y2+sn,pencolor)
.buffer.pset(x3+cs,y2-sn,pencolor)
.buffer.pset(x3-cs,y2-sn,pencolor)
end if
next angle
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

private:
sub drawsq9f(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
'MoveToEx(.Buffer.handle,x2,y2,0)
'LineTo(.Buffer.handle,x2,y3)
graphcursorpos2
'.Buffer.line(x3,0,x3,graph.height,pencolor)
'.Buffer.textout(x3,graphvspace+45,graphdateoncur,pencolor,&hffffff)
.Buffer.line(0,y3,graph.width,y3,pencolor)
'.Buffer.textout(0,y3,str$(graphpriceoncur),pencolor,&hffffff)
dim ydeg as double,sqfact as integer,stepdeg as integer,ydeg2 as double
sqfact=100
stepdeg=15
for i=1 to 24
if sq9diredit.text="1" then
ydeg=(sqr(graphpriceoncur*val(sq9edit.text))+stepdeg/360*i)^2/val(sq9edit.text)
end if
if sq9diredit.text="-1" then
ydeg=(sqr(graphpriceoncur*val(sq9edit.text))-stepdeg/360*i)^2/val(sq9edit.text)
end if
ydeg2=(graphvhigh-ydeg)/graphpricespace*graphvspace+graphvbegin
.Buffer.line(0,ydeg2,graph.width,ydeg2,pencolor)
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawtsq9f(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
graphcursorpos2
.Buffer.line(x3,0,x3,graph.height,pencolor)
dim ydeg as double,sqfact as integer,stepdeg as integer,ydeg2 as double
sqfact=100
stepdeg=15
dim savedtsq9fs as integer
savedtsq9fs=graphserialdateoncur
for i=1 to 24
if sq9diredit.text="1" then
ydeg=(sqr(graphjuliendateoncur*val(sq9edit.text))+stepdeg/360*i)^2/val(sq9edit.text)
end if
if sq9diredit.text="-1" then
ydeg=(sqr(graphjuliendateoncur*val(sq9edit.text))-stepdeg/360*i)^2/val(sq9edit.text)
end if
'ydeg2=(graphvhigh-ydeg)/graphpricespace*graphvspace+graphvbegin
DEFLNG yy, mm, dd,dayofweek
dim j as integer
dim sq9xval as integer
for j=savedtsq9fs to graphserialdateend
DayOfWeek = DateFromSerial(j, yy, mm, dd)
if int(ydeg)=juliandate(dd,mm,yy)+1 then
sq9xval=(j-graphserialdatebegin)/graphserialdatespace*graphhspace+graphhbegin
.Buffer.line(sq9xval,0,sq9xval,graph.height,pencolor)
savedtsq9fs=j
exit for
end if
next j
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawsq144(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
graphcursorpos2
.Buffer.line(0,y3,graph.width,y3,pencolor)
dim ydeg as double,sqfact as integer,stepdeg as integer,ydeg2 as double
sqfact=val(sq144edit.text)
stepdeg=1
for i=1 to 24
ydeg=graphpriceoncur/sqfact/4
ydeg=ydeg*5
if sq144diredit.text="1" then
ydeg=ydeg+i
end if
if sq144diredit.text="-1" then
ydeg=ydeg-i
end if
ydeg=ydeg/5
ydeg=ydeg*sqfact*4
ydeg2=(graphvhigh-ydeg)/graphpricespace*graphvspace+graphvbegin
.Buffer.line(0,ydeg2,graph.width,ydeg2,pencolor)
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawtsq144(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
graphcursorpos2
.Buffer.line(x3,0,x3,graph.height,pencolor)
dim ydeg as double,sqfact as integer,stepdeg as integer,ydeg2 as double
sqfact=val(sq144edit.text)
stepdeg=1
dim savedtsq9fs as integer
savedtsq9fs=graphserialdateoncur
for i=1 to 24
ydeg=graphjuliendateoncur/sqfact/4
ydeg=ydeg*5
if sq9diredit.text="1" then
ydeg=ydeg+i
end if
if sq9diredit.text="-1" then
ydeg=ydeg-i
end if
ydeg=ydeg/5
ydeg=ydeg*sqfact*4
'ydeg2=(graphvhigh-ydeg)/graphpricespace*graphvspace+graphvbegin
DEFLNG yy, mm, dd,dayofweek
dim j as integer
dim sq9xval as integer
for j=savedtsq9fs to graphserialdateend
DayOfWeek = DateFromSerial(j, yy, mm, dd)
if int(ydeg)=juliandate(dd,mm,yy)+1 then
sq9xval=(j-graphserialdatebegin)/graphserialdatespace*graphhspace+graphhbegin
.Buffer.line(sq9xval,0,sq9xval,graph.height,pencolor)
savedtsq9fs=j
exit for
end if
next j
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

end with

end sub

private:
sub drawpricecycles(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
.Buffer.line(x2,y2,x3,y3,pencolor)
'.Buffer.textout(x2-30,y2,"100",pencolor,&hffffff)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
.Buffer.line(x2,y2,x3,y2,pencolor)
.Buffer.line(x2,y3,x3,y3,pencolor)
'.Buffer.textout(x2-30,y3,"0",pencolor,&hffffff)
dim y4 as integer
for i=1 to 8
y4=y3+(y3-y2)*i
.Buffer.line(x2,y4,x3,y4,pencolor)
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub


private:
sub drawtimecycles(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
.Buffer.line(x2,y2,x3,y3,pencolor)
'.Buffer.textout(x2-30,y2,"100",pencolor,&hffffff)
dim a as double,b as double,c as double,ang_a as double
a=x3-x2
b=y3-y2
c=sqr(a^2+b^2)
if c<>0 then
ang_a=acos(a/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if
.Buffer.line(x2,y2,x2,y3,pencolor)
.Buffer.line(x3,y2,x3,y3,pencolor)
'.Buffer.textout(x2-30,y3,"0",pencolor,&hffffff)
dim x4 as integer
for i=1 to 8
x4=x3+(x3-x2)*i
.Buffer.line(x4,y2,x4,y3,pencolor)
next i
.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub


private:

sub drawlogspiral(xb,yb,xc,yc)
        
    defsng x2,y2,x3,y3   
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    
with qchart
.ClearBuffer
.buffer.bmp=.buffertmpz.bmp
dim a0 as double,b0 as double,c as double,ang_a as double
a0=x3-x2
b0=y3-y2
c=sqr(a0^2+b0^2)
if c<>0 then
ang_a=acos(a0/c)*180/3.1415927
dispinfotext("Angle: "+str$(ang_a)+chr$(10)+_
"Length: "+str$(c)_
+chr$(10)+"x1: "+str$(x2)_
+chr$(10)+"y1: "+str$(y2)_
+chr$(10)+"x2: "+str$(x3)_
+chr$(10)+"y2: "+str$(y3)_
)
end if

dim angrot as double
angrot=acos(a0/c)
dim distlogspi as double
if c>0 then
distlogspi=1/(abs(c))
else
distlogspi=0
end if

    dim b as double:b=DegToRad(80)
    dim a as double:a=sqr(distlogspi) 'rotation
    dim i as integer
    for i=0 to 500
        dim t as double:t=a*rqPI/4
        dim r as double:r = exp(t*cot(b+sqr(distlogspi)))
        points(i).r = r+x2
        points(i).x = r * cos(t)+x2
        points(i).y = r * sin(t)+y2
        a += 0.2
    next i
    for i=1 to 500
       .buffer.line(points(i-1).x,points(i-1).y,points(i).x,points(i).y,pencolor)
    next i



.buffertmp.bmp=.buffer.bmp
.paintchart
end with

end sub

'------------------------------------------------------------------------------------------
PRIVATE:
      SUB DrawPie (Overlay AS INTEGER)

       DIM i  AS INTEGER                               'Loop counters...
       DIM x1 AS LONG                                  'for readability
       DIM y1 AS LONG
       DIM x2 AS LONG
       DIM y2 AS LONG
       DIM Total AS SINGLE
       DIM pi2 AS SINGLE
       DIM pct AS SINGLE
       DIM Radius AS SINGLE
       DIM Cum AS SINGLE
       DIM xSize AS SINGLE
       DIM ySize AS SINGLE
       DIM xLeft AS LONG
       DIM xRight AS LONG
       DIM yTop AS LONG
       DIM yBottom AS LONG
       DIM TxtFactor AS SINGLE

       IF Overlay = FALSE THEN
        WITH QChart                                   'Set up the chart window
         .ClearBuffer                                'erase the current bitmap
         .DoLegend = True                            'Force to calculate sizes based on Legend
         .SetSize
         .SetSeries
         .DrawAxes
         .DrawTitles
         .DrawLegend
        END WITH
       END IF

       WITH QChart
        .Buffer.Font = .AxisFont
        TxtFactor = .Buffer.TextHeight("I")
        xSize = .XAxis.Len - TxtFactor
        ySize = .YAxis.Len - TxtFactor
        Total = .RowTotal(1)
        IF Total = 0 THEN
         MESSAGEDLG("Division by Zero!", 1, 4, 0)
         .ClearAll
         EXIT SUB
        END IF
        pi2 = 8*ATN(1)
        Radius = ySize / 2.5
        xLeft = (.XAxis.Ori + xSize / 2) - Radius
        xRight =(.XAxis.Ori + xSize / 2) + Radius
        yTop = (.YAxis.Ori - ySize / 2) - Radius
        yBottom = (.YAxis.Ori - ySize / 2) + Radius

        x1 = xRight                                   'Set Start point for 1st wedge
        y1 = (yTop + yBottom) \ 2
        Cum = 0
        FOR i = 1 TO .Data.ColCount
         IF .Data.Cell(i,1) <> "" THEN               'if data not missing

          Cum = Cum + VAL(.Data.Cell(i,1))          'Calculate wedge endpoint
          x2 = .XAXis.Ori + xSize / 2 + Radius * COS(pi2 * Cum / Total)
          y2 = .YAxis.Ori - ySize / 2 - Radius * SIN(pi2 * Cum / Total)

          IF .Series(i).HatchStyle = hsSolid THEN    'Set brush color for wedge
           .hBrush = CreateSolidBrush(.Series(i).Color)
          ELSE
           .hBrush = CreateHatchBrush(.Series(i).HatchStyle, .Series(i).Color)
          END IF
          .OldBrush = SelectObject(.Buffer.Handle, .hBrush)

          Pie (.Buffer.Handle, xLeft, yTop, xRight, yBottom, x1, y1, x2, y2)'Draw the wedge

          DeleteObject(SelectObject(.Buffer.Handle, .OldBrush))

          SELECT CASE .ChartStyle                   'Print labels at midpoint of wedge
          CASE csPiePct
           pct = VAL(.Data.Cell(i,1))/Total * 100
           .AxisLbl.Text = FORMAT$("%-3.1f", pct) + " %"

          CASE csPieVal
           .AxisLbl.Text = FORMAT$("%-4.4g", VAL(.Data.Cell(i,1)))

          CASE ELSE
           .AxisLbl.Text = ""
          END SELECT

          .AxisLbl.Left = .XAxis.Ori + xSize/2 + (Radius + TxtFactor)*_
           COS(pi2 * (Cum - VAL(.Data.Cell(i,1))/2) / Total)
          .AxisLbl.Top = .YAxis.Ori - ySize/2 - (Radius + TxtFactor)*_
           SIN(pi2 * (Cum - VAL(.Data.Cell(i,1))/2) / Total)
          IF COS(pi2 * (Cum - VAL(.Data.Cell(i,1))/2) / Total) < 0 THEN 'align left
           .AxisLbl.Left = .AxisLbl.Left - .Buffer.TextWidth(RTRIM$(.AxisLbl.Text))
          END IF
          .Buffer.TextOut (.AxisLbl.Left, .AxisLbl.Top, RTRIM$(.AxisLbl.Text),_
           .Buffer.Font.Color, .bgColor)
          x1 = x2                                   'Update wedge start position
          y1 = y2

         END IF
        NEXT i

       END WITH
       QChart.ChartExists = TRUE

      END SUB
'------------------------------------------------------------------------------------------
'------------------------- Public Methods -------------------------------------------------
'------------------------------------------------------------------------------------------
PUBLIC:
      SUB DrawChart(Overlay AS INTEGER)

       WITH QChart
        SELECT CASE .ChartType
        CASE ctBar
         .DrawBar(Overlay)
        CASE ctXY
         .DrawXY(Overlay)
        CASE ctLine
         .DrawLine(Overlay)
        CASE ctBox
         .DrawBox(Overlay)
        CASE ctPie
         .DrawPie(Overlay)
        END SELECT
        IF .Printing = FALSE THEN .PaintChart
       END WITH

      END SUB

public:
sub bufcntreset
bufcnt=0
end sub

public:
sub firstbufcntreset
firstbufcnt=0
end sub

public:
sub savebuffertmp
with qchart
if firstbufcnt=0 then
.firstbuffertmpz.bmp=.buffer.bmp
firstbufcnt++
end if
if bufcnt=0 then
.buffertmpz.bmp=.buffer.bmp
bufcnt++
end if
.buffertmp.bmp=.buffer.bmp
end with
end sub

public:
sub savebuffertmpsimple
with qchart
.buffertmp.bmp=.buffer.bmp
end with
end sub

public:
sub savebuffertmpsimplez
with qchart
.buffertmpz2.bmp=.buffer.bmp
end with
end sub

public:
sub restorebuffertmpz
with qchart
.buffer.bmp=.buffertmpz.bmp
end with
end sub

public:
sub restorefirstbuffertmpz
with qchart
.buffer.bmp=.firstbuffertmpz.bmp
end with
end sub

PUBLIC:
sub drawp(type,xb,yb,xc,yc,xd,yd)
defint x2,y2,x3,y3,x4,y4
defint type2
type2=type
'print type2
    
    x2=xb
    y2=yb
    x3=xc
    y3=yc
    x4=xd
    y4=yd
WITH QChart
select case type2
case 0:
.drawtrendline(x2,y2,x3,y3)
case 1:
.drawfibofan(x2,y2,x3,y3)
case 2:
.drawfiboret(x2,y2,x3,y3)
case 3:
.drawpara(x2,y2,x3,y3,x4,y4)
case 4:
.drawhline(x2,y2,x3,y3)
case 5:
.drawvline(x2,y2,x3,y3)
case 6:
.drawsqr(x2,y2,x3,y3)
case 7:
.drawtri(x2,y2,x3,y3)
case 8:
.drawcircle(x2,y2,x3,y3)
case 9:
.drawcross(x2,y2,x3,y3)
case 10:
.drawinvcircle(x2,y2,x3,y3)
case 11:
.drawaiming(x2,y2,x3,y3)
case 12:
.drawhand(x2,y2,x3,y3)
case 13:
.drawsin(x2,y2,x3,y3)
case 14:
.drawlog(x2,y2,x3,y3)
case 15:
.drawexp(x2,y2,x3,y3)
case 16:
.drawellipse(x2,y2,x3,y3)
case 17:
.drawpitchfork(x2,y2,x3,y3,x4,y4)
case 18:
.drawpriceext(x2,y2,x3,y3,x4,y4)
case 19:
.drawsq9f(x2,y2,x3,y3)
case 20:
.drawsq144(x2,y2,x3,y3)
case 21:
.drawtimeext(x2,y2,x3,y3,x4,y4)
case 22
.drawtsq9f(x2,y2,x3,y3)
case 23:
.drawtsq144(x2,y2,x3,y3)
case 24:
.drawpricecycles(x2,y2,x3,y3)
case 25:
.drawtimecycles(x2,y2,x3,y3)
case 26:
.drawlogspiral(x2,y2,x3,y3)
end select
end with
end sub
'-------------------------------------------------------------------------------------------
PUBLIC:
      SUB RedrawChart               '<--- Call this SUB from your FORM.OnResize EVENT SUB

       WITH QChart
        .Buffer.Width =  .Width                 'recalc dimensions
        .Buffer.Height = .Height
        IF .ChartExists THEN .DrawChart(FALSE)        'only redraw if QChart.ChartExists
       END WITH

      END SUB
'-------------------------------------------------------------------------------------------
PUBLIC:
      SUB PrintChart (Prn AS INTEGER, Orient AS INTEGER, Margin AS INTEGER,_
        Copies AS INTEGER, Colr AS BYTE)
  'Prints copies of graph to a full page with a user specified margin and orientation
  'Colr is Boolean: TRUE = Use color for printing, FALSE = Use B&W for printing

       DIM Source    AS QRECT
       DIM Page      AS QRECT
       DIM OldBW     AS BYTE

       IF QChart.ChartExists = FALSE THEN EXIT SUB           'Don't print nonexistent chart

       Printer.PrinterIndex = Prn
       Printer.Orientation = Orient

       WITH Page                                             'Set page size
        .Top = Margin
        .Left = Margin
        .Right = .Left + Printer.PageWidth - (Margin * 2)
        .Bottom = .Top + Printer.PageHeight - (Margin * 2)
       END WITH

       WITH QChart
        .MainFont.Size = .MainFont.Size * 3
        .SubFont.Size = .SubFont.Size * 3
        .AxisFont.Size = .AxisFont.Size * 3
        .LegendFont.Size = .LegendFont.Size * 3
        .MarkerSize = .MarkerSize * 3

        IF Orient = 0 THEN                                  'Portrait
         .Buffer.Width = 1920                              'Scale to page size
         .Buffer.Height = 2400
        ELSE                                                'Landscape
         .Buffer.Width = 2560
         .Buffer.Height = 1920
        END IF

        .Printing = TRUE                                    'Prevent screen redraw

        IF Colr = FALSE THEN
         OldBW = .BW                                       'Remember BW setting
         .BW = TRUE                                        'Change to BW for Printing
        END IF

        IF .XAxis.AutoScale = TRUE THEN                     'Keep same Axis Scales
         .XAxis.AutoScale = NOT .XAxis.AutoScale           'turn off Autoscale
        END IF
        IF .YAxis.AutoScale = TRUE THEN                     'Keep same Axis Scales
         .YAxis.AutoScale = NOT .YAxis.AutoScale           'turn off Autoscale
        END IF
        .DrawChart(FALSE)                                   'Redraw at Page Size
        IF Colr = FALSE THEN .BW = OldBW                     'Restore BW setting
       END WITH

       WITH Source
        .Top = 0
        .Left = 0
        .Right = .Left + QChart.Buffer.Width
        .Bottom = .Top + QChart.Buffer.Height
       END WITH

       WHILE Copies > 0
        DOEVENTS
        Printer.BeginDoc                                    'Print
        Printer.CopyRect(Page, QChart.Buffer, Source)
        Printer.EndDoc
        DEC(Copies)
       WEND

       WITH QChart
        .MainFont.Size = .MainFont.Size / 3                 'Restore to screen size
        .SubFont.Size = .SubFont.Size / 3
        .AxisFont.Size = .AxisFont.Size / 3
        .LegendFont.Size = .LegendFont.Size / 3
        .MarkerSize = .MarkerSize / 3
        IF .XAxis.AutoScale = -2 THEN                       'AutoScale was changed
         .XAxis.AutoScale = NOT .XAxis.AutoScale           'Change it back
        END IF
        IF .YAxis.AutoScale = -2 THEN                       'AutoScale was changed
         .YAxis.AutoScale = NOT .YAxis.AutoScale           'Change it back
        END IF
        .ReDrawChart                                        'Redraw at Screen size
        .Printing = FALSE                                   'Reset Flag
       END WITH

      END SUB
'-------------------------------------------------------------------------------------------
PUBLIC:
      SUB SaveChart (FileNam AS STRING)

qcShowSave:
       WITH QChart
        .dlgSave.Caption = "Save Graph"
        .dlgSave.Filter = "Bitmap Files|*.bmp|All Files|*.*"
        .dlgSave.FilterIndex = 1
        IF LEN(FileNam) <> 0 THEN
         IF INSTR(UCASE$(FileNam), ".BMP") = 0 THEN FileNam = FileNam + ".bmp"
         .dlgSave.FileName = FileNam
        ELSE
         .dlgSave.FileName = "*.bmp"
        END IF

        IF .dlgSave.Execute AND LEN(.dlgSave.FileName) <> 0 THEN
         IF FILEEXISTS(.dlgSave.FileName) THEN             'check if file exists
          DIM Response AS INTEGER                         'ask user for decision
          Response =  MESSAGEDLG("File Already Exists. Overwrite " + .dlgSave.FileName + "?",_
           0, 1 OR 2 OR 8, 0)      ' Y OR N OR Cancel
          SELECT CASE Response
          CASE 7                                        ' NO; don't overwrite, ask again
           GOTO qcShowSave                             '<--- Use of a local GOTO loop!
          CASE 2                                        'CANCEL;don't overwrite, don't ask again
           EXIT SUB
          END SELECT
         END IF

         DIM File AS QFILESTREAM
         File.Open(.dlgSave.FileName, 65535)               '65535 = fmCreate
         .Buffer.SaveToStream (File)                     'save the data
         File.Close
        END IF
       END WITH

      END SUB

'-------------------------------------------------------------------------------------------
     END TYPE'QChart
'----- End Object Definition ***************************************************************